/**
 * @file
 * @brief Architecture and system dependent stuff
 *
 * @author Kamil Toman <ktoman@email.cz>
 *
 */  
#ifndef _ECHELOG_ARCHDEP_H
#define _ECHELOG_ARCHDEP_H

/** @brief Single Unix conformance wanted. */
#ifndef _XOPEN_SOURCE
#define _XOPEN_SOURCE 500
#endif
/** @brief Extra Gnu Features. */
#ifndef _GNU_SOURCE
#define _GNU_SOURCE
#endif
/** @brief Extra features for SunOS. */
#ifndef __EXTENSIONS__
#define __EXTENSIONS__
#endif

/** @brief Extra macros for fixed length types */
#ifndef __STDC_LIMIT_MACROS
#define __STDC_LIMIT_MACROS
#endif

/* Misc definitions */

#ifdef __GNUC__
#       undef inline
#       define NONRET __attribute__((noreturn))
#       define UNUSED __attribute__((unused))
#       define PACKED __attribute__((packed))
#else
#       error This program requires the GNU C compiler.
#endif

/** @brief Our definition of null pointer. */
#ifndef NULL
#define NULL 0
#endif

/* Get system feature list */
#include "../../config.h"

#ifdef HAVE_FEATURES_H
#include <features.h>
#endif

#ifdef HAVE_SYS_FEATURE_TESTS_H
#include <sys/feature_tests.h>
#endif

/* We need this for smooth compilation */
#include <sys/types.h>

/* We may need extra typedefs for dumb header structure
   as presented for example in OpenBSD */
#ifndef HAVE_U_TYPES
#include "../../lib/gnu/u_types.h"
#endif

#ifdef HAVE_STRINGS_H
#include <strings.h>
#endif

#ifdef HAVE_GETOPT_LONG
#include <getopt.h> 
#else
#include "../../lib/gnu/getopt.h"
#endif

#ifndef __GNUC__
#ifndef DOXY_SKIP
  #define __attribute__(x)
#endif
#endif

#ifndef HAVE_SNPRINTF
#include "../../lib/gnu/str_stdlib.h"
#endif

#ifndef HAVE_STRDUP 
#include "../../lib/gnu/str_stdlib.h"
#endif

#ifndef HAVE_STRTOL
#include "../../lib/gnu/str_stdlib.h"
#endif

#ifndef HAVE_STRTOUL
#include "../../lib/gnu/str_stdlib.h"
#endif

#ifndef HAVE_GETUTENT
#include "../../lib/gnu/getutent.h"
#endif

#ifndef HAVE_BSEARCH
#include "../../lib/gnu/bsearch.h"
#endif

#if HAVE_FLOCK
#include "lib/flock_std.h"
#else
#include "lib/flock_svr4.h"
#endif

#ifdef HAVE_SYS_CDEFS_H
#include <sys/cdefs.h>
#endif

#undef HAVE_LOCAL_CREDENTIALS
#if !defined(HAVE_LINUX_CREDENTIALS)

#       if defined(HAVE_STRUCT_CMSGCRED) 
    	typedef struct cmsgcred tCred;
#       define cr_uid cmcred_uid
#       define HAVE_LOCAL_CREDENTIALS 1
        
#       elif defined(HAVE_STRUCT_FCRED) 
     	typedef struct fcred tCred;
#       define cr_uid fc_uid
#       define HAVE_LOCAL_CREDENTIALS 1

#       elif (defined(HAVE_STRUCT_SOCKCRED) && defined(LOCAL_CREDS))
        typedef struct sockcred tCred;
#       define cr_uid sc_uid
#       define HAVE_LOCAL_CREDENTIALS 1
#endif
#endif


/* 
 * Fixed size typedefs (C99 style)
 */

#ifdef HAVE_STDINT_H
#include <stdint.h>
#endif

#ifdef HAVE_INTTYPES_H
#include <inttypes.h>
#endif


#if defined(__STDC_LIMIT_MACROS) && !defined(INT32_MAX)
#include "../../lib/gnu/stdint_macros.h"
#endif

/** @brief Byte type definition. */
typedef unsigned char byte;

#ifndef DOXY_SKIP
#define INT8_T_BITS 8
#define INT8_T_BYTES (sizeof (int8_t)) 
#define INT16_T_BITS 16
#define INT16_T_BYTES (sizeof (int16_t))
#define INT32_T_BITS 32
#define INT32_T_BYTES (sizeof (int32_t))
#define INT64_T_BITS 64
#define INT64_T_BYTES (sizeof (int64_t))
#define UINT8_T_BITS 8
#define UINT8_T_BYTES (sizeof (uint8_t))
#define UINT16_T_BITS 16
#define UINT16_T_BYTES (sizeof (uint16_t))
#define UINT32_T_BITS 32
#define UINT32_T_BYTES (sizeof (uint32_t))
#define UINT64_T_BITS 64
#define UINT64_T_BYTES (sizeof (uint64_t))
#endif


#endif

