/**
 * @file
 * @brief Reading of configuration files (lexical analyzer).
 *
 * This file contains implementation of the lexical analyzer used for
 * configuration file reading. 
 *
 * @author Ales Privetivy <ales@ucw.cz>
 */

#ifndef _ECHELOG_CONFLEXER_H
#define _ECHELOG_CONFLEXER_H

/**
 * @brief Restriction on maximal size of the key data.
 */
#define MAXDATALEN                 2048

/**
 * @brief Restriction on maximal depth of the file nesting.
 */
#define CONF_MAX_INCLUDE_DEPTH     10

namespace lib {

#ifndef DOXY_SKIP
    namespace conflex_lib {

    /// structure for file nesting 
    struct tConfNest {
        /// parsing state of flex
        void * buff;
        /// filename on this level
        char * filename;
        /// number of line
        int line_no;
    };

    /* values of tokens */
    enum LX_RESULT {
        LX_EOF=0,       ///<  end of file 
        LX_SECTION=1,   ///<  section token
        LX_DATA=2,      ///<  data token for key
        LX_IDENT=3,     ///<  identifier of a key
        LX_ERROR=4      ///<  error
    };

    /* array for file nesting */
    extern struct tConfNest gb_conf_stack[CONF_MAX_INCLUDE_DEPTH];
    /* actual index in nest-array */
    extern int gb_include_stack_ptr;
    /* pointer to variable counting lines */
    extern int *gb_actual_line;
    /* error message (NULL -> no error)*/
    extern char *gb_conf_error;

    /* lexical element currently returned */
    extern char *gb_conf_data;

    extern char *gb_messagedef;

    /* lexical parser */
    extern int conflex();

    /* file preprocessor */
    extern int expandlex();
    }
}

/* We need to temporarily drop the namespace becouse 
 * input and output files wouldn't be recognized 
 * by flex otherwise
 */
extern FILE *confin, *confout;
extern FILE *expandin, *expandout;

#endif

#endif
