/**
 * @file
 * @brief Echelog strptime implementation.
 *
 * This code is here just for case we are not satisfied with the system's
 * strptime(). We should expect that the system wide strptime() can touch
 * unmatched values in <code>struct tm</code>. This NetBSD implementation does
 * not do it.
 *
 * @author Martin Macok <martin.macok@underground.cz>
 */

#ifndef _STRPTIME_H_
#define	_STRPTIME_H_

#include <time.h>
#include <limits.h>

namespace lib {

/**
 * @brief Convert a string representation of time to a time tm structure.
 *
 * This is  the  complementary  function  to  <tt>strftime()</tt> and 
 * converts the character string pointed to by <tt>buf</tt> to values
 * which are stored in the <tt>tm</tt> structure pointed to by <tt>tim</tt>,
 * using  the format specified by <tt>fmt</tt>.  Here format is a character
 * string that consists of field descriptors and text characters. For better
 * description of field descriptors see UNIX manpage <tt>strptime(3)</tt>. 
 *
 * @note Initialization of all fields of tm to INT_MAX before use is
 * recommended.
 *
 * @return The return value of the function is a pointer to the first character
 * not processed in this function call. In case the input string contains more
 * characters than required by the format string the return value points right
 * after the last consumed input  character.   In  case  the whole  input 
 * string  is  consumed  the  return value points to the NUL byte at the end
 * of the string.  If <tt>strptime()</tt> fails to match all of the format
 * string and therefore an error occurred the function returns NULL.
 *
 */
char *ech_strptime_p(const char *buf, const char *fmt, struct tm *tim);

}

#endif /* !_STRPTIME_H_ */
