/**
 * @file
 * @brief Echelog miniagent support library
 *
 * This class covers the message handling and a communication
 * with next echelog modules. It's primarily used by tMagent
 * for a low-level operations.
 *
 * @author Kamil Toman <ktoman@email.cz>
 *
 */

#ifndef  ECHELOG_MSGFRAME_HXX
#define  ECHELOG_MSGFRAME_HXX

#include "magents/msgframe.h"

namespace magents {

/** Reset the message frame for future reuse */
inline void tMsgFrame::reset ()
{
    /* Reset the message frame */
    message.rewind ();
    num_of_records = 0;
    incremental_message = false;
    length = empty_message_length;
}


/** Add a chunk to current message */
inline bool tMsgFrame::add_chunk (messages::tMsgChunk &c)
{
    using namespace lib;
    using namespace messages;
    int bytes_written;

    debug_log ('m', "Appending new chunk (%li) to message", c.get_type ());
    /* Store the value */
    if ((bytes_written = c.put (message.get_body ())) <= 0)
    {
        log (L_ERROR, "Can't store the chunk to message buffer");
        reset ();
        return true;
    }

    /* Update message statistics */
    length += bytes_written;
    num_of_records++;
    incremental_message = (c.get_status() != messages::ST_ITEM);

    return false;
}

}

#endif
