/**
 * @file
 * @brief Echelog miniagent support library
 *
 * This class implements an external command source 
 * buffering interface. It can be used to exec and gain 
 * the standard output of a system utility.
 *
 * @author Kamil Toman <ktoman@email.cz>
 *
 */

#ifndef ECHELOG_INFOBUFFER_HXX
#define ECHELOG_INFOBUFFER_HXX

#include "magents/infobuffer.h"

namespace magents {

/* This is actual tInfoBuffer::peek_byte() implementation */     
inline int tRunBuffer::peek_byte ()
{
    register char *c = cache_pos + 1;

    /* Advance / wrap around the cache border */ 
    if (c >= cache_start + CACHE_SIZE)
        c = cache_start;

    /* If there is no data to peek at then load them */
    if (c == cache_end && !data_available)
    {
        unsigned long orig = data_to_unget;
        long r = fetch ();

        if (r < 0) 
            return r;

        /* Adjust number of bytes which can 
         * be unget from the cache */
        data_to_unget -= r;
        if (data_to_unget > orig)
            data_to_unget = 0;

        data_available = true;
    }

    /* Return a byte (automatically) casted to int */
    return (static_cast<unsigned char> (*c));
}


/* This is actual tInfoBuffer::get_byte() implementation */     
inline int tRunBuffer::get_byte ()
{
    int r = peek_byte ();

    if (r > 0)
    {
        /* Advance / wrap around the cache border */ 
        if (++cache_pos >= cache_start + CACHE_SIZE)
            cache_pos = cache_start;
        data_to_unget++;
    }

    /* Success, return a byte (automatically) casted to int */
    data_available = false;
    return r;
}


/* This is actual tInfoBuffer::get_unbyte() implementation */     
inline int tRunBuffer::unget_byte ()
{
    /* Check if there are any data to be unget */
    if (!data_to_unget)
        return -1;

    /* Unget / wrap around the cache border */
    if (--cache_pos < cache_start)
        cache_pos = cache_start + CACHE_SIZE - 1;

    data_to_unget--;
    data_available = true;

    /* Successfully pushed back */
    return 0;
}

}

#endif
