/*
 * @brief Echelog Library -- Message buffer handling
 * This class covers the I/O message buffering. 
 *
 * @author Kamil Toman <ktoman@email.cz>
 *
 */

#ifndef __ECHELOG_MSGBUF_HXX
#define __ECHELOG_MSGBUF_HXX

#include "messages/msgbuf.h"
#include "messages/testbuf.h"

namespace messages {


inline const char * tMsgBuf::get_buffer() const
{ 
    return (! header)? 0 : header->get_buffer (); 
}


inline bool tMsgBuf::is_r_prepared () const 
{ 
    return (r_prepared); 
}


inline bool tMsgBuf::is_w_prepared () const 
{ 
    return (w_prepared); 
}


inline const int32_t tMsgBuf::get_declared_length () const
{ 
    return declared_length;
}


inline int32_t tMsgBuf::get_body_field_size() const
{   
    return INT32_T_BYTES; 
}
    

inline const int32_t tMsgBuf::get_max_body_len () const
{
    CHECK_PREPARED();

    return max_body_len;
}


inline const int32_t tMsgBuf::get_store_length () const
{ 
    CHECK_PREPARED();
    
    if (header->get_store_length () <= 0)
        return EE_BUF_GENERAL_ERROR;
    if (body->get_store_length () < 0)
        return EE_BUF_GENERAL_ERROR;

    return header->get_store_length()   /* size of header */
         + get_body_field_size()        /* size of body length field */
         + body->get_store_length ();   /* size of body */
}


inline int32_t tMsgBuf::get_len () const
{
    int32_t len;

    CHECK_PREPARED();
    if ((len = get_store_length ()) <= 0)
        return EE_BUF_GENERAL_ERROR;

    /* store length without msg length field */
    return len - header->get_total_msg_length_field_size();
}


inline int tMsgBuf::rewind ()
{
    using namespace lib;

    CHECK_PREPARED();
    debug_log ('m', "Message buffer rewind");

    if (!body)
        return EE_BUF_GENERAL_ERROR;

    body->rewind ();
    return SUCCESS;
}

}
#endif
