/**
 * @file
 * @brief  Echelog Library -- Message buffer header 
 * 
 * @author Kamil Toman <ktoman@email.cz>
 */

#ifndef __ECHELOG_MSGHEADER_HXX
#define __ECHELOG_MSGHEADER_HXX


namespace messages {

inline int32_t tMsgHeader::pure_header_size () const
{
    return  INT32_T_BYTES      // Message type
        + ID_LENGTH            // Agent identificator
        + ID_LENGTH            // Agent instance identificator
        + ID_LENGTH            // Host identificator
        + INT32_T_BYTES        // Flag bit-field
        + INT32_T_BYTES        // Timestamp
        + INT32_T_BYTES;       // Number of subrecords
}


inline const char * tMsgHeader::get_buffer() const 
{ 
    return (!buffer)? 0 : buffer->get_buffer (); 
}


inline bool tMsgHeader::is_r_prepared () const 
{ 
    return (r_prepared); 
}


inline bool tMsgHeader::is_w_prepared () const 
{ 
    return (w_prepared); 
}


inline int32_t tMsgHeader::get_header_length_field_size() const
{   
    return INT32_T_BYTES; 
}


inline int32_t tMsgHeader::get_total_msg_length_field_size () const 
{   
    return INT32_T_BYTES; 
}


inline int32_t tMsgHeader::get_store_length () const 
{
    ASSERT (get_len() >= 0);

    return get_len()                            // Pure header length
        + get_total_msg_length_field_size();    // Total message length
}


inline void tMsgHeader::set_flag_incremental (bool fl) 
{ 
    if (fl) 
        flags |= HDR_FLAG_INCREMENTAL; 
    else 
        flags &= ~(HDR_FLAG_INCREMENTAL); 
}


inline bool tMsgHeader::get_flag_incremental () const
{ 
    return flags & HDR_FLAG_INCREMENTAL; 
}


inline void tMsgHeader::set_flag_aggregable (bool fl) 
{ 
    if (fl) 
        flags |= (HDR_FLAG_AGGREGABLE); 
    else 
        flags &= ~(HDR_FLAG_AGGREGABLE); 
}


inline bool tMsgHeader::get_flag_aggregable () const 
{ 
    return flags & HDR_FLAG_AGGREGABLE; 
}


}

#endif
