/**
 * @file
 * @brief Echelog Library -- "Type" handling
 * This type can be used for handling single byte values 
 *      
 * @author Kamil Toman <ktoman@email.cz>
 *
 */
#ifndef _ECHELOG_BYTE_H
#define _ECHELOG_BYTE_H

#include "messages/types/echelogtype.h"

#include <stdio.h>

namespace messages {

class tByte : public tEchelogType
{
    protected:
        /** 
         * Property for basic type info/assignment 
         */
        byte value;      

        /** Get type info */
        virtual tTypeNumber get_type () const;

    public:
        /** The value is explicitly set null on creation */
        tByte(lib::tMemPool &mem_pool_buffer) 
            : tEchelogType (mem_pool_buffer), value (0)
            { }

        /** Create the "type" and assign the given value */
        tByte(lib::tMemPool &mem_pool_buffer, byte val) 
            : tEchelogType (mem_pool_buffer), value (val) 
            { len = 1; }

        /** The property is almost a simple one */
        inline byte get_value () const { return value; }
        inline void set_value (byte prop_value)
            { value = prop_value, len = 1; }
    
        /** We need a byte and another byte for its header */
        virtual int32_t get_store_length () { return len + 1; } 

        /** As always we have to at least implement loading from a buffer */
        virtual int32_t get (tPlainBuf *buf); 
        /** As always we have to at least implement storing to a buffer */
        virtual int32_t put (tPlainBuf *buf);

        /** Common debugging print outs */
        virtual long sprint (lib::tList &list, lib::tMemPool &pool, 
                const tFmtNode *node = NULL);
        
        /** Get the default formating tree */
        virtual const tFmtNode * get_default_format();

        /** Get the plain formating tree */
        virtual const tFmtNode * get_plain_format();

        /** Scan the string list */
        virtual int sscan (lib::tList &string_list);

        /** Type assignment */
        virtual tTYPE_RELATION assign (tEchelogType &tp);
        
        /** Type comparison '==' operator */
        virtual tTYPE_RELATION order (tEchelogType &tp);
};

}

#endif
