/**
 * @file
 * @brief Echelog Library -- "Type" handling
 * This type can be used for handling 32bit integers.
 *      
 * @author Kamil Toman <ktoman@email.cz>
 *
 */
#ifndef _ECHELOG_INT32_H
#define _ECHELOG_INT32_H

#include "messages/types/echelogtype.h"

namespace messages {

class tInt32 : public tEchelogType
{
    protected:
        /** 
         * Property for basic type info/assignment 
         */
        int32_t value;      

        /** Get type info */
        virtual tTypeNumber get_type () const;

    public:
        /** The value is explicitly set null on creation */
        tInt32(lib::tMemPool &mem_pool_buffer) 
            : tEchelogType (mem_pool_buffer), value (0) {}

        /** Create the "type" and assign the given value */
        tInt32(lib::tMemPool &mem_pool_buffer, int32_t val) 
            : tEchelogType (mem_pool_buffer), value (val) 
            { len = INT32_T_BYTES; }
        
        /** The property is almost a simple one */
        inline int32_t get_value () const { return value; }
        inline void set_value (int32_t prop_value)
            { value = prop_value, len = INT32_T_BYTES; }
    
        /** We need len bytes for an integer and one byte for its header */
        virtual int32_t get_store_length () { return len + 1; } 

        /** As always we have to at least implement loading from a buffer */
        virtual int32_t get (tPlainBuf *buf); 
        /** As always we have to at least implement storing to a buffer */
        virtual int32_t put (tPlainBuf *buf);

        /** Common debugging print outs */
        virtual long sprint (lib::tList &list, lib::tMemPool &pool, 
                const tFmtNode *node = NULL);

        /** Get the default formating tree */
        virtual const tFmtNode * get_default_format();

        /** Get the plain formating tree */
        virtual const tFmtNode * get_plain_format();
       
        /** Scan the string list */
        virtual int sscan (lib::tList &string_list);

        /** Type assignment */
        virtual tTYPE_RELATION assign (tEchelogType &tp);
        
        /** Type comparison '==' operator */
        virtual tTYPE_RELATION order (tEchelogType &tp);
};

}

#endif
