/**
 * @file
 * @brief Echelog Library -- "Type" handling
 * This type can be used for handling IPv4 addresses
 *      
 * @author Kamil Toman <ktoman@email.cz>
 *
 */
#ifndef _ECHELOG_IPV4ADDR_H
#define _ECHELOG_IPV4ADDR_H

#include "messages/types/int32.h"

namespace messages {

class tIPv4Addr : public tInt32
{
    protected:

        /** Get type info */
        virtual tTypeNumber get_type () const;

    public:
        /** The value is explicitly set null on creation */
        tIPv4Addr (lib::tMemPool &mem_pool_buffer) : tInt32 (mem_pool_buffer)
            { }

        /** Create the "type" and assign the given value */
        tIPv4Addr (lib::tMemPool &mem_pool_buffer, int32_t val) 
            : tInt32 (mem_pool_buffer, val) { }

        /** Common debugging print outs */
        virtual long sprint (lib::tList &list, lib::tMemPool &pool, 
                const tFmtNode *node = NULL);

        /** Get the default formating tree */
        virtual const tFmtNode * get_default_format();

        /** Get the plain formating tree */
        virtual const tFmtNode * get_plain_format();
       
        /** Scan the string list */
        virtual int sscan (lib::tList &string_list);

        /** Type assignment */
        virtual tTYPE_RELATION assign (tEchelogType &tp);
        
        /** Type comparison '==' operator */
        virtual tTYPE_RELATION order (tEchelogType &tp);
};

}

#endif
