/**
 * @file
 * @brief Echelog Library -- Echelog "Type" manager 
 * @author Ales Privetivy <ales@ucw.cz> 
 * @author Kamil Toman <ktoman@email.cz>
 *
 */

#ifndef _ECHELOG_TYPEMGR_H
#define _ECHELOG_TYPEMGR_H

#include "messages/typelist.h"

namespace messages {

namespace private_messages {

    struct tComponentDesc;
    
    /// description of one type
    struct tTypeDesc {
        /// numeric type id
        tTypeNumber type_number;
        /// name of type
        const char *name;
        /// flag if type is registered 
        int registered;
        /// number of components
        int32_t num_of_components;
        /// component descriptor table
        tComponentDesc *type_components;
    };

    /// component description
    struct tComponentDesc {
        /// name of component
        const char *name;
        /// echelog type name of component
        const char *type_name;
        /// link to subordinate component table
        tTypeDesc *type_descriptor;
    };
    
}


/// Type Manager class - provides type instances by name
class tTypeMgr {

    private:

        /// echelog typelist
        private_messages::tTypeDesc type_list[TOTAL_ECHELOG_TYPES];

    private:

        /// get a type description of 'name'
        private_messages::tTypeDesc * tTypeMgr::get_typedesc (const char *name)
            const;
        /// get component descriptor
        private_messages::tComponentDesc *tTypeMgr::get_component_desc 
            (private_messages::tTypeDesc &tp, const char* comp_name) const;
        /// get a component type by path and descriptor
        const char *get_component_type_by_desc_path 
            (private_messages::tTypeDesc &tp, lib::tList &path) const;

    public:

        /// type manager constructor
        tTypeMgr ();

        /// check if type is supported and marking it as available
        void register_type (const char *name);

        /// calls  register_type() for all types in list
        void register_type_list (class lib::tList *list);

        /// get one instance of given type
        tEchelogType * get_type_instance (lib::tMemPool &pool, const char *name)
            const;

        /// check if given type exists
        /// returns 0 if not registered, >0 otherwise
        int check_type (const char *name) const;

        /// Get a component
        const char *get_component_type_by_path (const char *type_name, 
                const lib::tList &path) const;

        /// Get a component
        const char *get_component_type (const char *type_name,
                const char *comp_name) const
            { lib::tMemPool p(1024); lib::tList l(p);
              if (s_strdsep (l, comp_name, ".", p) < 0)
                  return NULL;
              else
                  return get_component_type_by_path (type_name, l); }


};

/// global variable for type management (we need only one instance)
extern class tTypeMgr gb_type_manager;

}

#endif
