/**
 * @file
 * @brief Contains declaration of tConfigurable interface
 *
 * @author Marek Zoth <marek.zoth@volny.cz>
 *
 */

#ifndef ECHELOG_CONFIGURABLE_H
#define ECHELOG_CONFIGURABLE_H

#include "lib/lib.h"

namespace mframes {

/**
 * TODOC:
 * @brief Basic interface of the filter
 *
 * This is the description of the filter interface only. No functionality
 * is covered by this class. All instantiatable filters should be derived
 * directly or indirectly from this interface. Filter should implement all
 * pure virtual methods in the meaning as they are described here.
 */
class tConfigurable
{
    public:
        //-------------------------------
        // Interface
        //-------------------------------

        /**
         * @brief TODOC:
         */
        virtual bool register_sections (lib::tMemPool& trash_pool) = 0;

        /**
         * @brief TODOC:
         */
        virtual void unregister_sections () = 0;

        /**
         * @brief TODOC:
         */
        virtual bool process_config (lib::tMemPool& conf_mp,
                lib::tMemPool& trash_pool) = 0;

        /**
         * @brief TODOC:
         */
        virtual void cleanup_config () = 0;

        /**
         * @brief Constructor
         */
        inline tConfigurable () {};

        /**
         * @brief Destructor
         */
        virtual ~tConfigurable () {};                                                   
};

} //end of namespace mframes

#endif //ifdef ECHELOG_CONFIGURABLE_H

