/**
 * @file
 * @brief Contains declaration of tIFrame and tOFrame interfaces
 *
 * @author Marek Zoth <marek.zoth@volny.cz>
 *
 */

#ifndef  ECHELOG_IOFRAME_H
#define  ECHELOG_IOFRAME_H

#include "socks/unixsocket.h"
#include "msgtarget.h"

namespace mframes {

class tIFrame
{
    public:
        //-------------------------------
        // Interface
        //-------------------------------

        /// Closes the filter frame and destroys it
        virtual ~tIFrame () {};

        /**
         * @brief TODOC:
         */
        virtual bool connect_in () = 0;

        /**
         * @brief TODOC:
         */
        virtual void close_in () = 0;

        /**
         * @brief TODOC:
         */
        inline void set_target (tBufferTarget* target)
        { mp_target = target; }

        /**
         * @brief TODOC:
         */
        inline tBufferTarget* get_target ()
        { return mp_target; }

        /**
         * @brief TODOC:
         */
        static inline tIFrame* get_actual_iframe ()
        { return sp_actual_iframe; }

        /**
         * @brief TODOC:
         */
        static inline void set_actual_iframe (tIFrame* iframe)
        { sp_actual_iframe = iframe; }

    protected:
        /// Constructor, performs basic initialization
        tIFrame () {};

        //-------------------------------
        // Used only components
        //-------------------------------

        ///TODOC:
        tBufferTarget* mp_target;

        //-------------------------------
        // Owned components
        //-------------------------------

        /// Messages' raw data
        static char s_message_buffer[];

        /// IOFrame stored as actually used frame
        static tIFrame* sp_actual_iframe;
};

class tOFrame
{
    public:
        //-------------------------------
        // Interface types
        //-------------------------------

        class tOutput
        {
            public:
                //-------------------------------
                // Interface types
                //-------------------------------

                /**
                 * @brief Sends a message to output.
                 *
                 * @param message
                 * Message to be sent.
                 *
                 * @return
                 * On success returns false. On failure returns true.
                 */
                bool send_message (const messages::tMsgBuf& message);

                /**
                 * @brief TODOC:
                 *
                 * @return
                 * On success returns false. On failure returns true.
                 */
                virtual bool send_buffer (char* buf, int len) = 0;

                /// TODOC:
                inline const char* get_name () const
                { return m_name; }

            protected:
                //-------------------------------
                // Internal methods
                //-------------------------------

                /// Constuctor
                tOutput () : m_name (NULL) {};

                /// Destructor
                virtual ~tOutput () {};

                //-------------------------------
                // Used-only components
                //-------------------------------

                /// Output name
                const char* m_name;
        };

        //-------------------------------
        // Interface
        //-------------------------------

        /// Closes the filter frame and destroys it
        virtual ~tOFrame () {};

        /**
         * TODOC:
         * @brief Establishes connection of the outputs.
         *
         * @return
         * On success returns false. On failure returns true.
         */
        virtual bool connect_out () = 0;

        /**
         * TODOC:
         * @brief Output connections cleanup
         */
        virtual void close_out () = 0;

        /**
         * TODOC:
         * @brief Output's handle lookup method
         *
         * @param name
         * Name of the output which handle is requested.
         *
         * @return
         * Handle of the output for given name or invalid
         * handle if output of this name doesn't exist.
         */
        tOutput* get_output_by_name (const char* name = NULL) const;

        /**
         * @brief TODOC:
         */
        static inline tOFrame* get_actual_oframe ()
        { return sp_actual_oframe; }

        /**
         * @brief TODOC:
         */
        static inline void set_actual_oframe (tOFrame* oframe)
        { sp_actual_oframe = oframe; }

    protected:
        /// Constructor, performs basic initialization
        tOFrame ();

        //-------------------------------
        // Owned components
        //-------------------------------

        /// Number of output descriptions stored in m_outputs array
        int m_output_count;

        /// Array of outputs
        tOutput** m_outputs;

        /// IOFrame stored as actually used frame
        static tOFrame* sp_actual_oframe;
};


} //end of namespace mframes

#endif //ifdef ECHELOG_IOFRAME_H
