/**
 * @file
 * @brief Contains declaration of tSocketIO class
 *
 * @author Marek Zoth <marek.zoth@volny.cz>
 *
 */

#ifndef ECHELOG_SOCKETIO_H
#define ECHELOG_SOCKETIO_H

#include "configurable.h"
#include "socks/ctrl_unixsocket_nb.h"

namespace mframes {

/**
 * TODOC:
 */
class tSocketIO : public tConfigurable
{
    public:
        //-------------------------------
        // Interface
        //-------------------------------

        /**
         * TODOC:
         * @brief Filter module's initialization
         *
         * @return
         * On success returns false. On failure returns true.
         */
        bool init (volatile bool* socket_error);

        /**
         * TODOC:
         * @brief Filter module's cleanup
         *
         * Call this function to clean and release all resources. Should
         * be called after successful initialization by init().
         */
        virtual void cleanup ();

        /**
         * @brief Destructor
         */
        virtual ~tSocketIO ();

        /**
         * @brief TODOC:
         */
        void register_echelog_module (const char* mod_name);

        /**
         * TODOC:
         * @brief Frame's message processing method
         *
         * Call this to let the frame do its work ie.
         * reading the message and pass it to the filter.
         */
        bool process (int timeout = -1);

        /**
         * @brief TODOC:
         */
        inline void set_sock_err ()
        { *mp_socket_error = true; }

        /**
         * @brief TODOC:
         */
        static inline tSocketIO* get_actual_socketio ()
        { return sp_actual_sockio; }

        /**
         * @brief TODOC:
         */
        static inline void set_actual_socketio (tSocketIO* sockio)
        { sp_actual_sockio = sockio; }

        //-------------------------------
        // Implementation of tConfigurable interface
        //-------------------------------

        /**
         * @brief TODOC:
         */
        virtual bool register_sections (lib::tMemPool& trash_pool);

        /**
         * @brief TODOC:
         */
        virtual void unregister_sections ();

        /**
         * @brief TODOC:
         */
        virtual bool process_config (lib::tMemPool& conf_mp,
                lib::tMemPool& trash_pool);

        /**
         * @brief TODOC:
         */
        virtual void cleanup_config ();

    protected:
        /**
         * @brief TODOC:
         */
        struct tOutputInfo
        {
            const char* name;
            const char* path;
        };

        /**
         * @brief Constructor
         */
        tSocketIO ();

        inline bool is_initialized ()
        { return m_siostate & SIO_INITIALIZED; }

        //-------------------------------
        // Owned components
        //-------------------------------

        /// Filter frame socket manager
        socks::tSocketMgr m_smgr;

        /// Path for input UD-socket
        char* m_input_name;

        /// Holds info about outputs parsed from configuration
        tOutputInfo* m_oia;

        /// Number of output infos in m_oia array
        int m_oia_count;

        //-------------------------------
        // Used only components
        //-------------------------------

        /// Flag to be set on socket error
        volatile bool* mp_socket_error;

    private:
        /**
         * @brief TODOC:
         */
        static const int SIO_CREATED;
        static const int SIO_CONFIGURED;
        static const int SIO_INITIALIZED;

        //-------------------------------
        // Object state attributes
        //-------------------------------

        /// Actual state of filter module
        int m_siostate;

        //-------------------------------
        // Owned components
        //-------------------------------

        /// Control unix socket for echelog daemon
        socks::tCtrlUNIXSocketNB m_ctrl;

        /// Description of "System" section in configuration
        lib::cfitem* m_system_section;

        //-------------------------------
        // Used-only components
        //-------------------------------

        /// IOFrame stored as actually used frame
        static tSocketIO* sp_actual_sockio;

};

} //end of namespace mframes

#endif //ifdef ECHELOG_SOCKETIO_H

