/**
 * @file
 * @brief Contains declaration of tUNIXSocketIFrame, tUNIXSocketOFrame
 * tUNIXSocketIOFrame classes
 *
 * @author Marek Zoth <marek.zoth@volny.cz>
 *
 */

#ifndef  ECHELOG_COMMONIOFRAME_H
#define  ECHELOG_COMMONIOFRAME_H

#include "socks/unixsocket.h"
#include "mframes/ioframe.h"
#include "mframes/socketio.h"

namespace mframes {

class tUNIXSocketIFrame : public tIFrame, virtual public tSocketIO
{
    public:
        //-------------------------------
        // Interface
        //-------------------------------

        virtual bool connect_in ();

        virtual void close_in ();

        /// Constructor, performs basic initialization
        tUNIXSocketIFrame ();

    private:
        //-------------------------------
        // Internal methods
        //-------------------------------

        /// Socket hangup handler
        static void hup_f(socks::tSocket *sock);

        /// Socket error handler
        static void err_f(socks::tSocket *sock);

        /// Processes incoming data
        static void process_f (socks::tSocket *sock);

        //-------------------------------
        // Object state attributes
        //-------------------------------

        /// true if successfully connected
        bool m_connected;

        //-------------------------------
        // Owned components
        //-------------------------------

        /// Input unix socket
        socks::tUNIXSocket m_input;
};

class tUNIXSocketOFrame : public tOFrame, virtual public tSocketIO
{
    public:
        //-------------------------------
        // Interface
        //-------------------------------

        /// Constructor, performs basic initialization
        tUNIXSocketOFrame ();

        /**
         * TODOC:
         * @brief Establishes connection of the outputs.
         *
         * @return
         * On success returns false. On failure returns true.
         */
        virtual bool connect_out ();

        /**
         * TODOC:
         * @brief Output connections cleanup
         */
        virtual void close_out ();

    private:
        //-------------------------------
        // Internal types
        //-------------------------------

        class tOutputInt : public tOutput
        {
            public:
                /**
                 * TODOC:
                 */
                inline socks::tUNIXSocket& get_socket ()
                { return m_output; }

                /**
                 * TODOC:
                 */
                inline void set_name (const char* name)
                { m_name = name; }

                /**
                 * TODOC:
                 */
                virtual bool send_buffer (char* buf, int len);

            private:
                /// Output unix socket
                socks::tUNIXSocket m_output;
        };

        //-------------------------------
        // Internal methods
        //-------------------------------

        /// Socket error handler
        static void s_err_f(socks::tSocket *sock);

        //-------------------------------
        // Object state attributes
        //-------------------------------

        /// true if successfully connected
        bool m_connected;
};

class tUNIXSocketIOFrame : public tUNIXSocketIFrame, public tUNIXSocketOFrame
{
    public:
        /// Constructor, performs basic initialization
        tUNIXSocketIOFrame () {};
};

} //end of namespace mframes

#endif //ifdef ECHELOG_COMMONIOFRAME_H
