/**
 * @file
 * @brief Control command layer
 *
 * @author Kamil Toman <ktoman@email.cz>
 *
 */

#ifndef _ECHELOG_CONTROL_HXX
#define _ECHELOG_CONTROL_HXX

#include "lib/lib.h"

namespace socks {


inline int tControl::test_retries ()
{
    if (retry_count-- > 0)
        return -1;

    /* Okay, we do have a next chance */
    return 0;
}


inline int tControl::test_received_cmd () const
{
    using namespace private_socks;
    tCmdHeader *header = static_cast<tCmdHeader *> (cmd_in_buffer);

    /*
     * Check if we operate on the same protocol version and if the
     * supplied length field makes sence.
     */
    if (header->protocol_version != ECHELOG_CTRL_PROTOCOL_VERSION ||
            header->ad_len < 0 || 
            header->ad_len > static_cast<long> ((MAX_CMD_DATA_LENGTH)))
        return -1;

    /* Seems OK */
    return 0;
}
   

inline int tControl::test_received_cmdtype (tCMD_TYPE command) const
{
    using namespace private_socks;
    using namespace lib;

    tCmdHeader *header = static_cast<tCmdHeader *> (cmd_in_buffer);

    /* Check received message for correctness */
    if (test_received_cmd ())
        return -1;

    /* And if it does conform to the specified command type */
    if (header->command != command)
        return -1;


    /* Seems OK */
    return 0;
}

    
}

#endif
