/**
 * @file
 * @brief Echelog Control Commands Supplementary
 *
 * @author Kamil Toman <ktoman@email.cz>
 *
 */

#ifndef _ECHELOG_CTRL_CMD_HXX
#define _ECHELOG_CTRL_CMD_HXX

#include "lib/lib.h"
#include <string.h>


namespace socks {


inline bool cmd_is_privileged (const tCMD_TYPE type)
{
    return type & 0xFF000000;
}


inline bool cmd_is_system (const tCMD_TYPE type)
{
    return type & 0x00F00000;
}


inline bool cmd_needs_confirmation (const tCMD_TYPE type)
{
    return type & 0x00000F00;
}


inline tCommand * create_simple_command (const tCMD_TYPE command)
{
    tCommand *c = static_cast<tCommand *> (lib::xmalloc (sizeof (tCommand)));

    /* Just create a command of the specified type with no ancillary data */
    c->command = command;
    c->d_len = 0;
    c->d_ptr = NULL;

    return c;
}


inline void free_cmd (tCommand *c)
{
    using namespace lib;

    /* Release the command structure completely */
    if (c)
    {
        xfree (c->d_ptr);
        xfree (c);
    }
}

}


#endif
