/**
 * @file
 * @brief Non-blocking UNIX domain sockets
 *
 * @author Kamil Toman <ktoman@email.cz>
 *
 */

#ifndef ECHELOG_SOCKET_NB_HXX
#define ECHELOG_SOCKET_NB_HXX

#include "lib/lib.h"
#include <unistd.h>
#include <fcntl.h>

namespace socks {

/// Stop reading operation
inline void tSocketNB::stop_r_io ()
{
    r_io.last_op = IO_OP_NO_OPERATION;
    if (last_operation == OP_IO && w_io.last_op == IO_OP_NO_OPERATION)
            last_operation = OP_NO_OPERATION;
}

/// Stop writing operation
inline void tSocketNB::stop_w_io ()
{
    w_io.last_op = IO_OP_NO_OPERATION;
    if (last_operation == OP_IO && r_io.last_op == IO_OP_NO_OPERATION)
            last_operation = OP_NO_OPERATION;
}

/// Clear both io operations
inline void tSocketNB::clear_io ()
{
    r_io.last_op = w_io.last_op = IO_OP_NO_OPERATION;
}

}

#endif

