/*
 * @brief Echelog Spool Library -- Pagefile implementation - header file
 * expansion pack
 * @author Ales Privetivy <ales@ucw.cz>
 *
 */

/** forces synchro of mmaped file */
inline int tPageFile::synchro_pagefile()
{
    using namespace lib;

    /* if pagefile is not maped no synchro is needed */
    if ( !present ) return 0;

    /* we do synchro only if we are asked to */
    if ( !write_through ) return 0;

    /* do the synchronization */
    if ( msync ( pg_data, pg_max_page_size, MS_SYNC ) )
    {
        log(L_ERROR, "PAGEFILE: Cannot synchronize pagefile (\"%s\"): %m",
	             filename );
        return -1;
    }

    return 0;
}

/* returns maximal size of item which will be able to fit into the
 * pagefile */
inline int tPageFile::get_free_space_size()
{
    /* if pagefile is not maped we do not know anything about it */
    if (!present) return 0;

    /* calculate the free space */
    return pg_max_page_size - pg_index[last_id - first_id];
}

/* checks if need bytes can be still stored in pagefile and whether
 * we have free entry for it */
inline int tPageFile::is_space(int length)
{
    using namespace lib;

    /* check on item count limit */
    if ( last_id > last_id + 1 )
    {
         log(L_ERROR, "PAGEFILE: Limit on item identificator number reached!");
         return 0;
    }

    /* check on item size */
    if ( length + (int)sizeof(struct tItem) <= get_free_space_size() &&
        pg_items < pg_max_items ) return 1; else return 0;
}

/* returns the pointer to item with item_id */
inline char *tPageFile::get_item(tIdentificator item_id)
{
    /* check if it in range of this pagefile */
    if ( item_id < first_id )
        return NULL;
    if ( item_id >= last_id )
        return NULL;

    /* return pointer to item data */ 
    return ( pg_data + pg_index[item_id - first_id ] + sizeof( struct tItem ));
}

/* marks the pagefile as full */
inline void tPageFile::set_closed()
{
    closed = 1;
}

/* if the file is no longer needed and can be removed */
inline int tPageFile::to_remove()
{
    if (closed && pg_nonextracted == 0) return 1; else return 0;
}

/* returns the item description structure */
inline struct tItem *tPageFile::get_item_description(tIdentificator item_id, struct tItem *it_desc)
{
    /* check if it in range of this pagefile */
    if ( item_id < first_id )
          return NULL;
    if ( item_id >= last_id )
          return NULL;

    /* return pointer to item data */
    return static_cast<struct tItem *>
	   ( memcpy(it_desc, pg_data+pg_index[item_id-first_id], 
		    sizeof(struct tItem)));
}
