/**
 * @brief Echelog Library -- Preprocessor for configuration files 
 * @author Ales Privetivy <ales@ucw.cz>
 *      
 */


%option noyywrap
%option nounput
%option prefix="expand"

%{
     #include <string.h>
     #include <stdio.h>
     #include <ctype.h>
     #include "lib/lib.h"
     #include "lib/conflexer.h"

     namespace lib {
         
     /* Some local stuff */
    
     static char * cfil,*dfil,*cfl,*efil;
     static char * CE_DEEP = "includes nested too deeply";
     static char * CE_NOTEXISTS = "file for inclusion does not exists";

     }
%}

IDENT       [_a-zA-Z][_A-Za-z0-9]*
ESCAPE      \\[^\n]
EOF         <<EOF>>

%%
        using namespace lib;
        using namespace conflex_lib;

[ \t]*include[ \t]*\"({ESCAPE}|[^\n\"\\])*\"[ \t]*$  {


                // check if maximal nesting depth reached
                if ( gb_include_stack_ptr+1 >= CONF_MAX_INCLUDE_DEPTH )
                {
                    fclose ( yyout );
                    fclose ( yyin );
                    for (int aa = gb_include_stack_ptr - 1; aa >= 0; aa--)
                    {
                        yy_switch_to_buffer( (YY_BUFFER_STATE)
                                gb_conf_stack[aa].buff );
                        fclose( yyin );
                        if (aa > 0)
                            yy_delete_buffer( YY_CURRENT_BUFFER );
                    }

                    gb_conf_error = CE_DEEP;
                    return -1;
                }

                /* look for left " */
                cfil = strchr (yytext, '"');
                cfil++;

                /* look for right " */
                dfil = strrchr (cfil, '"');
                *dfil = 0;

                /* strip slashes */
                for(efil=dfil=cfil;*dfil;dfil++)
                {
                    if (*dfil=='\\') 
                        if (dfil[1]) dfil++; 
                    *efil ++= *dfil;         
                }
                *efil = 0;

                // backup current buffer
                gb_conf_stack[gb_include_stack_ptr].buff = YY_CURRENT_BUFFER;
                gb_include_stack_ptr++;

                if (*cfil == '/')
                {
                    // absolute pathname
                    cfl = gb_conf_stack[gb_include_stack_ptr].filename = 
                        (char *)cf_malloc(strlen(cfil)+1);
                    strcpy (cfl, cfil);
                }
                else
                {
                    // extract filename in which the new file is included
                    dfil = gb_conf_stack[gb_include_stack_ptr-1].filename;
                    // allocate enough space for new file-name
                    cfl = gb_conf_stack[gb_include_stack_ptr].filename = 
                        (char *)cf_malloc(strlen(cfil)+strlen(dfil)+1);

                    // duplicate the name
                    strcpy(cfl,dfil); 

                    // leave only path from it
                    dfil = strrchr (cfl, '/');
                    if (dfil)
                        *(++dfil) = 0;
                    else
                        dfil = cfl, *dfil = 0;

                    // append new relative pathname
                    strcpy (dfil, cfil);
                }

                // add the info about included  to stack
                gb_conf_stack[gb_include_stack_ptr].line_no = 1; 
                gb_actual_line = &(gb_conf_stack[gb_include_stack_ptr].line_no);

                // try to open it
                FILE *incl_in = fopen (cfl, "r");

                // file not exist or another problem ?
                if ( ! incl_in )
                {
                    fclose ( yyout );
                    for (int aa = gb_include_stack_ptr - 1; aa >= 0; aa--)
                    {
                        yy_switch_to_buffer( (YY_BUFFER_STATE)
                                gb_conf_stack[aa].buff );
                        fclose( yyin );
                        if (aa > 0)
                            yy_delete_buffer( YY_CURRENT_BUFFER );
                    }

                    gb_conf_error = CE_NOTEXISTS; 
                    return -1;
                }

                fprintf(yyout, "\n");
                for(int i = 0; i <= gb_include_stack_ptr; i++)
                    fprintf(yyout, "# %s:%d\n", 
                           gb_conf_stack[i].filename, gb_conf_stack[i].line_no);
                fprintf(yyout, "\n");

                // switch the processing to it
                yy_switch_to_buffer( 
                    yy_create_buffer( incl_in, YY_BUF_SIZE ) ); 

}


[^\n]*           {
                                fprintf( yyout, "%s", yytext );
                 }

\n               {
                                // line counter
                                (*gb_actual_line)++;
                                fprintf( yyout, "\n" );
                 }


       
<<EOF>>          {
                                // we are at end of last file
                                if ( --gb_include_stack_ptr < 0 )
                                {
                                        fclose( yyin );
                                        fclose( yyout );
                                        return 0; 
                                }
                                else
                                {
                                        // leaving nested file
                                        fclose( yyin );
                                        yy_delete_buffer( YY_CURRENT_BUFFER );
                                        yy_switch_to_buffer( (YY_BUFFER_STATE)
                                             gb_conf_stack[gb_include_stack_ptr].buff );
                                        gb_actual_line = &(gb_conf_stack[gb_include_stack_ptr].line_no);

                                        fprintf(yyout, "\n");
                                        for(int i=0;i<=gb_include_stack_ptr;i++)
                                            fprintf(yyout, "# %s:%d\n",
                                                    gb_conf_stack[i].filename, gb_conf_stack[i].line_no);
                                        fprintf(yyout, "\n");

                                }

               }

%%

namespace lib {
    namespace conflex_lib {
        /* Namespace wrapper function */
        int expandlex (void) { return ::yylex(); }
    }
}
