/**
 * @brief  Echelog Library -- Lexical parser for time strings
 * @author Ales Privetivy <ales@ucw.cz>
 *      
 */

%option noyywrap
%option nounput
%option prefix="parsetime"
%option case-insensitive

%{

#include "lib/archdep.h"
#include <stdio.h>
#include "lib/lib.h"
#include "parsetime_parser.h"

using namespace lib;

namespace lib {

char *gb_parsetimestr;

#define YY_INPUT(buf,result,max_size) \
{ \
  result = (*gb_parsetimestr == 0) ? YY_NULL : (buf[0] = *gb_parsetimestr++, 1); \
}

}
%}

NUMBER	        [0-9]*

%%

[\ \t\n]*
"+"             return '+';
"-"             return '-';
"/"             return '/';
"."             return '.';
":"             return ':';
start           return START;
end             return END;
now             return NOW;
am              return AM;
pm              return PM;
yesterday       return YESTERDAY;
today           return TODAY;
tomorrow        return TOMORROW;
{NUMBER}        {
                    parsetimelval.NumericValue = atol(yytext);
                    return NUMBER;
                }
jan             { 
                    parsetimelval.NumericValue = 0;
                    return MONTH_NAME;
                }
feb             {
                    parsetimelval.NumericValue = 1;
                    return MONTH_NAME;
                }
mar             {
                    parsetimelval.NumericValue = 2;
                    return MONTH_NAME;
                }
apr             {
                    parsetimelval.NumericValue = 3;
                    return MONTH_NAME;
                }
may             {
                    parsetimelval.NumericValue = 4;
                    return MONTH_NAME;
                }
jun             {
                    parsetimelval.NumericValue = 5;
                    return MONTH_NAME;
                }
jul             {
                    parsetimelval.NumericValue = 6;
                    return MONTH_NAME;
                }
aug             {
                    parsetimelval.NumericValue = 7;
                    return MONTH_NAME;
                }
sep             {
                    parsetimelval.NumericValue = 8;
                    return MONTH_NAME;
                }
oct             {
                    parsetimelval.NumericValue = 9;
                    return MONTH_NAME;
                }
nov             {
                    parsetimelval.NumericValue = 10;
                    return MONTH_NAME;
                }
dec             {
                    parsetimelval.NumericValue = 11;
                    return MONTH_NAME;
                }
january         {
                    parsetimelval.NumericValue = 0;
                    return MONTH_NAME;
                }
february        {
                    parsetimelval.NumericValue = 1;
                    return MONTH_NAME;
                }
march           {
                    parsetimelval.NumericValue = 2;
                    return MONTH_NAME;
                }
april           {
                    parsetimelval.NumericValue = 3;
                    return MONTH_NAME;
                }
june            {
                    parsetimelval.NumericValue = 5;
                    return MONTH_NAME;
                }
july            {
                    parsetimelval.NumericValue = 6;
                    return MONTH_NAME;
                }
august          {
                    parsetimelval.NumericValue = 7;
                    return MONTH_NAME;
                }
september       {
                    parsetimelval.NumericValue = 8;
                    return MONTH_NAME;
                }
october         {
                    parsetimelval.NumericValue = 9;
                    return MONTH_NAME;
                }
november        {
                    parsetimelval.NumericValue = 10;
                    return MONTH_NAME;
                }
december        {
                    parsetimelval.NumericValue = 11;
                    return MONTH_NAME;
                }
sun             {
                    parsetimelval.NumericValue = 0;
                    return DAY_OF_WEEK;
                }
mon             {
                    parsetimelval.NumericValue = 1;
                    return DAY_OF_WEEK;
                }
tue             {
                    parsetimelval.NumericValue = 2;
                    return DAY_OF_WEEK;
                }
wed             {
                    parsetimelval.NumericValue = 3;
                    return DAY_OF_WEEK;
                }
thu             {
                    parsetimelval.NumericValue = 4;
                    return DAY_OF_WEEK;
                }
fri             {
                    parsetimelval.NumericValue = 5;
                    return DAY_OF_WEEK;
                }
sat             {
                    parsetimelval.NumericValue = 6;
                    return DAY_OF_WEEK;
                }
sunday          {
                    parsetimelval.NumericValue = 0;
                    return DAY_OF_WEEK;
                }
monday          {
                    parsetimelval.NumericValue = 1;
                    return DAY_OF_WEEK;
                }
tuesday         {
                    parsetimelval.NumericValue = 2;
                    return DAY_OF_WEEK;
                }
wednesday       {
                    parsetimelval.NumericValue = 3;
                    return DAY_OF_WEEK;
                }
thursday        {
                    parsetimelval.NumericValue = 4;
                    return DAY_OF_WEEK;
                }
friday          {
                    parsetimelval.NumericValue = 5;
                    return DAY_OF_WEEK;
                }
saturday        {
                    parsetimelval.NumericValue = 6;
                    return DAY_OF_WEEK;
                }
seconds         {
                    parsetimelval.NumericValue = 1;
                    return TIME_UNIT;
                }
second          {
                    parsetimelval.NumericValue = 1;
                    return TIME_UNIT;
                }
sec             {
                    parsetimelval.NumericValue = 1;
                    return TIME_UNIT;
                }
s               {
                    parsetimelval.NumericValue = 1;
                    return TIME_UNIT;
                }
minutes         {
                    parsetimelval.NumericValue = 2;
                    return TIME_UNIT;
                }
minute          {
                    parsetimelval.NumericValue = 2;
                    return TIME_UNIT;
                }
min             {
                    parsetimelval.NumericValue = 2;
                    return TIME_UNIT;
                }
m               {
                    parsetimelval.NumericValue = 2;
                    return TIME_UNIT;
                }
hours           {
                    parsetimelval.NumericValue = 3;
                    return TIME_UNIT;
                }
hour            {
                    parsetimelval.NumericValue = 3;
                    return TIME_UNIT;
                }
hr              {
                    parsetimelval.NumericValue = 3;
                    return TIME_UNIT;
                }
h               {
                    parsetimelval.NumericValue = 3;
                    return TIME_UNIT;
                }
days            {
                    parsetimelval.NumericValue = 4;
                    return TIME_UNIT;
                }
day             {
                    parsetimelval.NumericValue = 4;
                    return TIME_UNIT;
                }
d               {
                    parsetimelval.NumericValue = 4;
                    return TIME_UNIT;
                }
weeks           {
                    parsetimelval.NumericValue = 5;
                    return TIME_UNIT;
                }
week            {
                    parsetimelval.NumericValue = 5;
                    return TIME_UNIT;
                }
wk              {
                    parsetimelval.NumericValue = 5;
                    return TIME_UNIT;
                }
w               {
                    parsetimelval.NumericValue = 5;
                    return TIME_UNIT;
                }
months          {
                    parsetimelval.NumericValue = 6;
                    return TIME_UNIT;
                }
month           {
                    parsetimelval.NumericValue = 6;
                    return TIME_UNIT;
                }
years           {
                    parsetimelval.NumericValue = 7;
                    return TIME_UNIT;
                }
year            {
                    parsetimelval.NumericValue = 7;
                    return TIME_UNIT;
                }
yr              {
                    parsetimelval.NumericValue = 7;
                    return TIME_UNIT;
                }
y               {
                    parsetimelval.NumericValue = 7;
                    return TIME_UNIT;
                }
[^\ \t\n\+\-\.\/\:\s\m\h\d\w\y]	{
                    return ERROR;
         	} 

%%
