/**
 * @brief  Echelog Library -- Lexical parser for message definition 
 * @author Ales Privetivy <ales@ucw.cz>
 *      
 */

%option noyywrap
%option nounput
%option prefix="messagedef"

%{

#include "lib/archdep.h"
#include <stdio.h>
#include "lib/lib.h"
#include "lib/conflexer.h"
#include "messages/msgmgr.h"
#include "message_def_parser.h"

using namespace lib;
using namespace conflex_lib;

namespace lib {
    namespace conflex_lib {
        char *gb_messagedef;
    }
}

#define YY_INPUT(buf,result,max_size) \
{ \
  result = (*gb_messagedef == 0) ? YY_NULL : (buf[0] = *gb_messagedef++, 1); \
}


%}

IDENT           [_a-zA-Z][_A-Za-z0-9]*
NUMBER	        [0-9]*

%%

[\ \t\n]*
"("             return '(';
")"             return ')';
","             return ',';
";"             return ';';
"="             return '=';
":"             return ':';
"*"             return '*';
message_name	return MSG_NAME;
message_id		return MSG_ID;
message_attributes	return MSG_ATTRIBUTES;

{IDENT}         {
    				messagedeflval.StringValue = cf_strdup(yytext);
	    			return IDENT;
                }

{NUMBER}        {
		    		sscanf(yytext,"%ld",&messagedeflval.NumericValue);
			    	return NUM;
			    }

[^\ \t\n\(\)\,\;\=]	{
				return ERROR;
		    	} 

%%
