/**
 * @brief Echelog Library -- Syntax parser for message definition 
 * @author Ales Privetivy <ales@ucw.cz>
 *      
 */

%{

#include "lib/archdep.h"
#include <stdlib.h>
#include <stdio.h>

#include "lib/lib.h"
#include "lib/conflexer.h"
#include "messages/msgmgr.h"

using namespace lib;
using namespace conflex_lib;
using namespace messages;
using namespace private_messages;

%}

%union {
    long NumericValue;      /**<  numbers */
    char *StringValue;      /**<  strings, identifiers */
    struct messages::private_messages::tAttributeDefinition *AttrDef;
    class lib::tList *AttrList;
    struct messages::private_messages::tAttrBag AttrBag;
    struct messages::private_messages::tMsgDefinition *MsgDef;
}

%{

static tMsgDefinition *message_definition;
static struct tNode const *cc;

static char *gb_error_msg;
extern int yylex();
extern FILE *messagedefin,*messagedefout;

int yyparse();

int yyerror (char *s)
{
    gb_error_msg = s;
    return 0;
}

namespace messages {

    char * cf_register_message(struct cfitem *item, char *buffer)
    {
        using namespace lib;

        gb_messagedef = buffer;
        messagedefin = NULL;
        messagedefout = NULL;

        if (yyparse())
            return "syntax error when parsing RegisterMessage key";

        if (!message_definition->message_name)
            return "value of message_name must be set in RegisterMessage key";

        if (message_definition->message_type==-1)
            return "value of message_type must be set in RegisterMessage key";

        if (message_definition->attr_count==-1)
            return "value of message_attributes must be set in "
                   "RegisterMessage key";

        static_cast<class tList *>(item->data)->append(message_definition);

        return NULL; 
    }

    void cf_unregister_messages (struct cfitem *item)
    {
        static_cast<class tList *>(item->data)->reset ();
    }
}

%}

%start message_def 

%token ':'
%token '('
%token ')'
%token '='
%left  ';'
%left  ','
%token MSG_NAME
%token MSG_ID
%token MSG_ATTRIBUTES
%token <NumericValue> NUM
%token <StringValue> IDENT
%token ERROR

%type <AttrDef> attr_def
%type <AttrList> attr_list
%type <AttrBag> attributes_command
%type <NumericValue> id_command
%type <StringValue> name_command
%type <MsgDef> command_list message_def

%%

message_def:        command_list	{ 
                        $$=$1; message_definition= $$ 
                                    } 
;

command_list:                       {
                        $$ = static_cast<struct tMsgDefinition*>
                            (cf_malloc(sizeof(struct tMsgDefinition)));
                        $$->message_name = NULL;
                        $$->message_type = -1;
                        $$->attribute = static_cast
                            <struct tAttributeDefinition *>(NULL);
                        $$->attr_count = -1;
                                    } 
                    | name_command ';' command_list  {

                        $3->message_name = $1;
                        $$ = $3;
                                    }
                    | id_command ';' command_list {
                        $3->message_type = $1;
                        $$ = $3;
                                    }
                    | attributes_command ';' command_list {
                        $3->attr_count = $1.count;
                        $3->attribute = $1.al;
                        $$ = $3;
                                    }
;


name_command:       MSG_NAME '=' IDENT {
                          $$ = $3;
                                    }
;

id_command:         MSG_ID '=' NUM  {
                        $$ = $3;	
                                    }
;

attributes_command: MSG_ATTRIBUTES '=' '(' attr_list ')' {
                        int32_t i;
                        $$.count = $4->count();
                        $$.al = static_cast<struct tAttributeDefinition*>
                                (cf_malloc(sizeof(struct tAttributeDefinition)
                                *$$.count));

                        i = 0;
                        FOR_ALL_LIST_ITEMS (cc, $4)
                        {
                                $$.al[i++] = *static_cast
                                            <struct tAttributeDefinition*>
                                            (cc->data);
                        }
                                    }
;

attr_list:          attr_def    	{ 
                        $$ = new tList(get_cfpool ());
                        $$ -> prepend( $1 );
                                    }	 
                    | attr_def ',' attr_list {
                        $3 -> prepend( $1 );
                        $$ = $3;
                                    }
;

attr_def:           IDENT ':' IDENT {
                        $$ = static_cast<struct tAttributeDefinition*>
                             (cf_malloc(sizeof(struct tAttributeDefinition)));	
                        $$->attr_name = $1;
                        $$->type_name = $3;
                        $$->key = false;
                                    }
                    | '*' IDENT ':' IDENT {
                        $$ = static_cast<struct tAttributeDefinition*>
                             (cf_malloc(sizeof(struct tAttributeDefinition)));	
                        $$->attr_name = $2;
                        $$->type_name = $4;
                        $$->key = true;
                                    }
;

%%
