/**
 * @brief Echelog Library -- "Type" handling
 * This type can be used for handling single byte values
 *      
 * @author Kamil Toman <ktoman@email.cz>
 *
 */

#include "lib/lib.h"
#include "messages/testbuf.h"
#include "messages/types/byte.h"

namespace messages {

int32_t tByte::get (tPlainBuf *buf)
{
    byte tmp;

    CHECK_BUFFER (buf);
    READ_BYTE (buf, tmp);
    if (tmp)
    {
        len = static_cast<int32_t> (tmp);
        if (len != sizeof(byte))
            return EE_BUF_READ_ERROR;
        READ_BYTE (buf, value);
    }
    else
        len = 0;
     
    return len + 1;
}


int32_t tByte::put (tPlainBuf *buf)
{
    CHECK_BUFFER (buf);
    STORE_BYTE (buf, static_cast<byte> (len));

    if (len)
        STORE_BYTE (buf, value);

    return len + 1;
}

const tFmtNode * tByte::get_default_format()
{
    static tFmtNode * node = create_fmt_tree (get_prpool (), 
            "\"byte %c\"");
    return node;
}

const tFmtNode * tByte::get_plain_format()
{
    static tFmtNode * node = create_fmt_tree (get_prpool (), 
            "\"%c\"");
    return node;
}

long tByte::sprint (lib::tList &list, lib::tMemPool &pool, const tFmtNode *node)
{
    using lib::s_asprintf;

    /* return null pointer for null object */
    if (is_null ())
    {
        list.append (NULL);
        return 0;
    }

    /* use defaults for an empty tree */
    if (!node)
    {
        /* create a default tree in printout temporary pool */
        node = get_default_format ();
        ASSERT (node);

        /* sprint if to given pool */
        return sprint (list, pool, node);
    }

    ASSERT (node->num_of_nodes == 0);
    char *ret_str, *format = node->nodes[0].format;

    ASSERT (format);
    last_printout_len = s_asprintf (&ret_str, pool, format, value);
    list.append (ret_str);

    return last_printout_len;
}


int tByte::sscan (lib::tList &string_list)
{
    using namespace lib;

    if (!string_list.count())
        return 0;

    const tNode *c = string_list.get_first();
    char *s = static_cast<char *> (c->data);

    if (s)
    {
        long v;

        if (s_stol (s, &v, 0, UCHAR_MAX))
        {
            len = 0;
            return 0;
        }

        len = sizeof (byte);
        value = static_cast<byte> (v);
    }
    else
        len = 0;

    return 1;
}


tTYPE_RELATION tByte::assign (tEchelogType &tp)
{
    TEST_ASSIGN ();

    tByte &tpc = static_cast<tByte &> (tp);

    len = tpc.len;
    value = tpc.value;

    return TP_IDENTICAL;
}


tTYPE_RELATION tByte::order (tEchelogType &tp)
{
    COMMON_ORDER();

    tByte &tpc = static_cast<tByte &> (tp);
    NUMERIC_ORDER();
}

}
