/**
 * @file
 * @brief Echelog Library -- "Type" handling
 * This type can be used for handling "complex" echelog types
 *      
 * @author Kamil Toman <ktoman@email.cz>
 *
 * This file is generated automatically.
 */

#include "messages/testbuf.h"
#include "messages/typelist.h"

#define MIN_PRINTOUT_GUESS 50

/* Complex-defs are expanded below */ 

namespace messages {


tUser::tUser (lib::tMemPool &mem_pool_buffer)
  : tComplex (mem_pool_buffer),
    uid(mem_pool_buffer),
    username(mem_pool_buffer)
{
    components_names = static_cast<const char **> (mem_pool_buffer.alloc 
        (2 * sizeof (const char *)));
    components = static_cast<tEchelogType **> (mem_pool_buffer.alloc
        (2 * sizeof (tEchelogType *)));

    components[0] = &uid;
    components_names[0] = "uid";
    components[1] = &username;
    components_names[1] = "username";
}
    

const tFmtNode * tUser::get_default_format()
{
    static tFmtNode * node = create_fmt_tree (get_prpool (), "{\
            \"uid: %u\",\
            default }");
    return node;
}

}

namespace messages {


tHost::tHost (lib::tMemPool &mem_pool_buffer)
  : tComplex (mem_pool_buffer),
    ip(mem_pool_buffer),
    fqdn(mem_pool_buffer)
{
    components_names = static_cast<const char **> (mem_pool_buffer.alloc 
        (2 * sizeof (const char *)));
    components = static_cast<tEchelogType **> (mem_pool_buffer.alloc
        (2 * sizeof (tEchelogType *)));

    components[0] = &ip;
    components_names[0] = "ip";
    components[1] = &fqdn;
    components_names[1] = "fqdn";
}
    

const tFmtNode * tHost::get_default_format()
{
    static tFmtNode * node = create_fmt_tree (get_prpool (), "{\
            default,\
            default }");
    return node;
}

}

namespace messages {


tLogin::tLogin (lib::tMemPool &mem_pool_buffer)
  : tComplex (mem_pool_buffer),
    user(mem_pool_buffer),
    host(mem_pool_buffer),
    time(mem_pool_buffer)
{
    components_names = static_cast<const char **> (mem_pool_buffer.alloc 
        (3 * sizeof (const char *)));
    components = static_cast<tEchelogType **> (mem_pool_buffer.alloc
        (3 * sizeof (tEchelogType *)));

    components[0] = &user;
    components_names[0] = "user";
    components[1] = &host;
    components_names[1] = "host";
    components[2] = &time;
    components_names[2] = "time";
}
    

const tFmtNode * tLogin::get_default_format()
{
    static tFmtNode * node = create_fmt_tree (get_prpool (), "{\
            default,\
            default,\
            default }");
    return node;
}

}

