/**
 * @file
 * @brief Echelog Library -- Type Manager Class
 * get_typ_instance() method 
 *
 * @author Kamil Toman <ktoman@email.cz>
 *
 * This file is generated automatically.
 */

#include "lib/archdep.h"
#include <string.h>

#include "lib/lib.h"
#include "messages/types/typemgr.h"
#include "messages/typelist.h"

namespace messages {


using namespace lib;


/**
 * simple string hash function 
 */
static int hashfn (const char *type_name)
{
    int len = 0, sum = 0;
    const char *c;
    
    ASSERT (type_name != NULL);

    for (c = type_name; *c != 0; c++, len++)
        sum += *c;

    return 1000 * len + sum;
}


/**
 * This function should return a newly allocated class associated to "type"
 *
 * Input: name  -- name of "type"
 * Output: pointer to newly allocated instance of associated class 
 */
tEchelogType * tTypeMgr::get_type_instance(lib::tMemPool &pool, 
    const char *name) const
{
    int32_t hashcode = hashfn (name);

    switch (hashcode)
    {
        case 4308:
            if (!strcmp("BYTE", name) && type_list[0].registered)
                return new tByte (pool);
            break;
        case 4318:
            if (!strcmp("HOST", name) && type_list[1].registered)
                return new tHost (pool);
            break;
        case 4319:
            if (!strcmp("USER", name) && type_list[8].registered)
                return new tUser (pool);
            break;
        case 5336:
            if (!strcmp("INT32", name) && type_list[4].registered)
                return new tInt32 (pool);
            break;
        case 5338:
            if (!strcmp("INT16", name) && type_list[3].registered)
                return new tInt16 (pool);
            break;
        case 5377:
            if (!strcmp("LOGIN", name) && type_list[6].registered)
                return new tLogin (pool);
            break;
        case 7519:
            if (!strcmp("VARCHAR", name) && type_list[10].registered)
                return new tVarchar (pool);
            break;
        case 8607:
            if (!strcmp("HOSTNAME", name) && type_list[2].registered)
                return new tHostname (pool);
            break;
        case 8608:
            if (!strcmp("USERNAME", name) && type_list[9].registered)
                return new tUsername (pool);
            break;
        case 8627:
            if (!strcmp("UNIXTIME", name) && type_list[7].registered)
                return new tUnixTime (pool);
            break;
        case 9669:
            if (!strcmp("IPV4_ADDR", name) && type_list[5].registered)
                return new tIPv4Addr (pool);
    }

    die("Trying to get instance of unsupported type: %s",name);
}

}

