/**
 * @file
 * @brief Contains implementation of tIFrame, tOFrame, tOutput classes
 *
 * @author Marek Zoth <marek.zoth@volny.cz>
 *
 */

#include "lib/lib.h"
#include "mframes/ioframe.h"


namespace mframes {

//-------------------------------
// Implementation of tIFrame
//-------------------------------

/* static members definition */
char tIFrame::s_message_buffer[MAX_MESSAGE_SIZE];
tIFrame* tIFrame::sp_actual_iframe;

//-------------------------------
// Implementation of tOutput
//-------------------------------

bool tOFrame::tOutput::send_message (const messages::tMsgBuf& message)
{
    using namespace lib;

    /* pass the message */
    const long len = message.get_declared_length ();
    char* buffer = const_cast<char *> (message.get_buffer ());

    debug_log ('1', "Passing message (%i bytes) to output: %s",
        len, m_name);

    return send_buffer (buffer, len);
}


//-------------------------------
// Implementation of tOFrame
//-------------------------------

/* static members definition */
tOFrame* tOFrame::sp_actual_oframe;


/* Set up a filter frame and fill in basic information */
tOFrame::tOFrame ()
{
    m_output_count = 0;
    m_outputs = NULL;
}


tOFrame::tOutput* tOFrame::get_output_by_name (const char* name) const
{
    /* if no output is configured return NULL */
    if (m_output_count <= 0)
        return NULL;

    /* if the name is NULL then return first output */
    ASSERT (m_outputs != NULL);
    if (name == NULL)
        return *m_outputs;

    /* lookup handle for the output of given name */
    int i = 0;
    for (; i < m_output_count; i++)
        if (!strcmp (name, m_outputs[i]->get_name ()))
            break;

    if (i < m_output_count)
        return m_outputs[i];
    else
        return NULL;
}


} //end of namespace filters
