/**
 * @file
 * @brief Contains implementation of tMsgProcessingModule class
 *
 * @author Marek Zoth <marek.zoth@volny.cz>
 *
 */

#include "mframes/msgprocessingmodule.h"

namespace mframes {

using namespace lib;

/* Basic MsgProcessor initialization.*/
tMsgProcessingModule::tMsgProcessingModule (tCmdLine& cmdline, const char* cfg_filename)
    : tEchelogModule (cmdline, cfg_filename)
{
    mp_ioframe = NULL;
    m_inited = false;
}


/* MsgProcessor's initialization */
bool tMsgProcessingModule::init (tBufferTarget* target, bool enable_out)
{
    ASSERT (!m_inited);
    ASSERT (target != NULL);

    /* create io frame */
    tUNIXSocketIOFrame* iofr = NULL;
    tUNIXSocketIFrame* ifr = NULL;
    if (enable_out)
    {
        iofr = new tUNIXSocketIOFrame ();
        mp_ioframe = iofr;
    }
    else
    {
        ifr = new tUNIXSocketIFrame ();
        mp_ioframe = ifr;
    }

    if (mp_ioframe == NULL)
    {
        log (L_ERROR, "Not enough memory.");
        goto fail;
    }
    mp_ioframe->set_target (target);

    /* and call my ancestor's init */
    if (tEchelogModule::init (mp_ioframe, mp_ioframe,
            enable_out ? iofr : NULL))
        goto iof_clean;

    m_inited = true;

    return false;

iof_clean:
    delete mp_ioframe;
    mp_ioframe = NULL;
fail:

    return true;
}


/* MsgProcessor's cleanup */
void tMsgProcessingModule::cleanup ()
{
    ASSERT (m_inited);

    /* call my ancestor's cleanup */
    tEchelogModule::cleanup ();

    /* release the ioframe */
    delete mp_ioframe;
    mp_ioframe = NULL;

    m_inited = false;
}


/* Main MsgProcessor's working method */
int tMsgProcessingModule::process ()
{
    ASSERT (m_inited);

    /* process data on sockets */
    if (mp_ioframe->process ())
        return PR_BREAK_ERR;
    else
        return PR_CONTINUE;
}

} //end of namespace mframes

