/**
 * @file
 * @brief Contains implementation of tSimpleModule class
 *
 * @author Marek Zoth <marek.zoth@volny.cz>
 *
 */

#include "mframes/simplemodule.h"

namespace mframes {

using namespace lib;

/* Basic module initialization.*/
tSimpleModule::tSimpleModule (tCmdLine& cmdline, const char* cfg_filename)
    : tModule (cmdline, cfg_filename)
{
    m_inited = false;
}


bool tSimpleModule::init ()
{
    ASSERT (!m_inited);

    /* call ancestor's init */
    if (tModule::init ())
        goto fail;

    /* redirect the logging */
    if (log_file (m_log_name))
    {
        log (L_ERROR, "Error while redirecting log file to %s.",
            m_log_name ? m_log_name : "NULL");
    }
    m_inited = true;

    return false;

fail:
    return true;
}


void tSimpleModule::cleanup ()
{
    ASSERT (m_inited);

    /* call ancestor's cleanup */
    tModule::cleanup ();
    m_inited = false;
}


bool tSimpleModule::run ()
{
    ASSERT (m_inited);

    return process ();
}

} //end of namespace mframes

