/**
 * @brief Echelog Socks library -- Compression support
 *
 * Compress and decompress functions based on zlib calls.
 *
 * @author Ales Privetivy <ales@ucw.cz>
 *
 */

#include "lib/lib.h"
#include <zlib.h>

namespace socks {

using namespace lib;

int compress_data(void *buf, unsigned long *length)
{
     Bytef *tmpbuf;
     unsigned long out_len = *length;

     // we accept only rich customers !!!
     if (*length < 128) 
         return -1;

     // allocate output buffer memory
     if ((tmpbuf = static_cast<Bytef *>(xmalloc( 11 * *length/10 )))==NULL)
     {
         debug_log('l', "COMPRESS: Not enough memory for output buffer.");
         return -1;
     }

     // try to compress it
     if(compress (tmpbuf, &out_len, static_cast<Bytef *>(static_cast<void *>
                     (buf)), *length)==Z_OK && out_len<*length)
     {
         memcpy( buf, tmpbuf, out_len);
         *length = out_len;
         xfree( tmpbuf );
         return 0;
     }

     // at least we tried :)
     xfree( tmpbuf );
     return -1;
}


int uncompress_data(void *buf, unsigned long length, unsigned long *out_len)
{
     Bytef *tmpbuf;

     // allocate output buffer memory
     if ((tmpbuf = static_cast<Bytef *>(xmalloc( *out_len )))==NULL)
     {
         debug_log('l', "COMPRESS: Not enough memory for output buffer.");
         return -1;
     }

     // try to compress it
     if(uncompress (tmpbuf, out_len, static_cast<Bytef *>(static_cast<void *>
                   (buf)), length)==Z_OK)
     {
         memcpy(buf, tmpbuf, *out_len);
         xfree( tmpbuf );
         return 0;
     }

     // at least we tried :)
     xfree( tmpbuf );
     return -1;
}

}
