/**
 * @file
 * @brief Echelog Socks library -- Echelog Control Command Layer
 *
 * @author Kamil Toman <ktoman@email.cz>
 *
 */

#include "lib/lib.h"
#include "socks/ctrl_unixsocket.h"
#include <string.h>


namespace socks {

using namespace lib;    


/* Socket hangup/err handler during module registration */
void abort_f(class tSocket *sock)
{
    tSocketMgr *m = sock->get_smgr ();

    /* Print the error message and cleanup the socket */
    if (m) 
        if (m->unregister_socket (sock) < 0)
            log (L_ERROR, "Can't unregister socket (fd=%ld)", sock->get_fd());
    if (sock->close_socket())
        log(L_ERROR,"Error closing the control socket.");

    log(L_INFO,"Control connection closed.");
}


int tCtrlUNIXSocket::register_echelog_module (tSocketMgr &smanager,
        const char *name, long max_registration_time)
{
    set_ctrl_name (name);
    ASSERT (ctrl_name);

    /* Try to connect to overseer */
    if (create_client (DEFAULT_CTRL_SOCKET))
        return -1;

    /* Register the socket in order to immediately detect if the
     * other side hangs up */
    smanager.register_socket (this);

    /* Send an authentication request */
    store_cmd (CMD_IDENTIFY, static_cast<const void *> (ctrl_name),
            ctrl_namelen);
    if (send_cmd () < 0)
        return -1;

    /* And receive the reply */
    if (receive_cmd () < 0)
        return -1;

    /* Check if the authentication was successful */
    if (test_received_cmdtype (CMD_LOGIN_OK) < 0)
        return -1;
    
    /* Prepared to communicate with overseer */
    return 0;
}


void tCtrlUNIXSocket::setup_defaults (const char *name)
{
    /* Name the connection channel after the module name */
    set_ctrl_name (name);

    /* Abort if something goes wrong */
    set_on_data_hup (abort_f);
    set_on_data_err (abort_f);
}


int tCtrlUNIXSocket::send_cmd ()
{
    using namespace private_socks;

    tCmdHeader *h = static_cast<tCmdHeader *> (cmd_out_buffer);

    /* Send control command to the socket */
    return at_write (cmd_out_buffer, sizeof (tCmdHeader) + h->ad_len);
}


int tCtrlUNIXSocket::receive_cmd ()
{
    /* Receive control command from the socket */
    return at_read (cmd_in_buffer, MAX_CMD_LENGTH);
}


}
