/**
 * @file
 * @brief Echelog Binary storing utility
 *
 * This agent waits on specified UNIX-domain socket and stores
 * incomming messages in binary format.
 *
 * @author Ales Privetivy <ales@ucw.cz>
 *
 */                                                                             

#ifndef ECHELOG_BINSTOR_H
#define ECHELOG_BINSTOR_H

#include "mframes/msgtarget.h"
#include "spool/binstore.h"

/**
 * @brief TODOC:
 */
class tBinstorTarget : public mframes::tBufferTarget
{
    public:

        tBinstorTarget (): tBufferTarget() {};

        /**
         * @brief Initialization of the filter
         *
         * This method prepares filter for use with pass_msg().
         * It reads configuration and creates and prepares rules.
         *
         * @return
         * If succesful returns false. On failure return true.
         */
        virtual bool init ();

        /**
         * @brief Cleanup of the filter
         *
         * Releases and resets all allocated resources in init().
         * It is the reverse function for init().
         */
        virtual void cleanup ();

        /**
         * @brief Buffer processing function
         *
         * The function processes the received buffer <tt>buf</tt>
         * of given length <tt>len</tt>. It retrieves the message,
         * performs some basic checks and stores it to the configured
         * binary storage. Invalid messages are dropped.
         *
         * @return
         * On success returns false. On failure returns true.
         */
        virtual bool process_buffer (char* buf, int len);

        //------------------------------------------
        // Implementation of tConfigurable interface
        //------------------------------------------

        /**
         * @brief TODOC:
         */
        virtual bool register_sections (lib::tMemPool& trash_pool);

        /**
         * @brief TODOC:
         */
        virtual void unregister_sections ();

        /**
         * @brief TODOC:
         */
        virtual bool process_config (lib::tMemPool& conf_mp, 
                lib::tMemPool& trash_pool);

        /**
         * @brief TODOC:
         */
        virtual void cleanup_config ();

    private:
        /// Description of binstor's configuration
        static struct lib::cfitem s_binstor_section[];
};

extern class spool::tBinStore *bs;
extern char *index_file, *data_file;

#endif 
