/**
 * @file
 * @brief Header file for Echelog BinCut
 *
 * @author Ales Privetivy <ales@ucw.cz>
 *
 */                                                                             

#ifndef ECHELOG_BINCUT_H
#define ECHELOG_BINCUT_H

#include "mframes/simplemodule.h"
#include "spool/binstore.h"
#include "messages/msgbuf.h"
#include "messages/msgheader.h"

#include <sys/types.h>
#include <sys/stat.h>

/**
 * This class implements a typical miniagent's functionality.
 * The agent periodically reads informational chunks
 * from a supplied parser and forms them into messages which
 * are passed to the next module. It also covers the basic miniagent's
 * configuration and reconfiguration.
 *
 * This class should not have multiple instances in the program.
 */
class tBinCutModule : public mframes::tSimpleModule 
{
    public:
        tBinCutModule (tCmdLine& cmdline, const char* cfg_filename):
            tSimpleModule (cmdline, cfg_filename) {};

    protected:
        virtual bool process ();

        virtual bool register_sections (lib::tMemPool& trash_pool);

        virtual void unregister_sections ();

        virtual bool process_config (lib::tMemPool& conf_mp, lib::tMemPool&
                trash_pool);

        virtual void cleanup_config ();

    private:

        static char *s_input_data_file, *s_input_index_file, *s_output_data_file,
             *s_output_index_file, *s_starttime, *s_endtime;

        static struct lib::cfitem s_bincut_section[];

        /* starting and ending times of interval to cut */
        time_t m_start_time, m_end_time;
        /* output and input binstore parameters */
        class spool::tBinStore *m_outbin, *m_inbin;

};

#endif
