/**
 * @file
 * @brief Header file for Echelog BinGrep
 *
 * @author Ales Privetivy <ales@ucw.cz>
 *
 */                                                                             

#ifndef ECHELOG_BINGREP_H
#define ECHELOG_BINGREP_H

#include "mframes/simplemodule.h"
#include "spool/binstore.h"
#include "messages/msgbuf.h"
#include "messages/msgheader.h"
#include "messages/msgchunk.h"
#include "messages/msgmgr.h"

#include <sys/types.h>
#include <sys/stat.h>
#include <regex.h>

/* variables read from config file */
extern char *input_data_file, *input_index_file, *output_file,
            *attr_search, *pattern, *msgname, *hostname,
            *agentname, *instancenm;

/* list of attributes we will search in */
extern class lib::tList attrs_list;
/* list of messages we will search in */
extern class lib::tList msgs_list;
/* list of hosts we will search in */
extern class lib::tList hosts_list;
/* list of instances we will search in */
extern class lib::tList instances_list;
/* list of agents we will search in */
extern class lib::tList agents_list;

/* flag - search in all attributes */
extern int all_attrs;
/* flag - search in all msgs */
extern int all_msgs;
/* flag - search in all hosts */
extern int all_hosts;
/* flag - search in all instances */
extern int all_instances;
/* flag - search in all agents */
extern int all_agents;

/* input bin */
extern class spool::tBinStore *inbin;
/* regular expression tree */
extern regex_t    r_tree;

/* output instance to xml */
extern class lib::tXMLOutput xml;
/* file descriptor of output file */
extern int xml_fd;

/// print error output
void xml_die(char *message);

/// print error output for regexp
void xml_regexp_die(char *message);

void generate_output ();

/**
 * This class implements a typical miniagent's functionality.
 * The agent periodically reads informational chunks
 * from a supplied parser and forms them into messages which
 * are passed to the next module. It also covers the basic miniagent's
 * configuration and reconfiguration.
 *
 * This class should not have multiple instances in the program.
 */
class tBinGrepModule : public mframes::tSimpleModule
{
    public:
        tBinGrepModule (tCmdLine& cmdline, const char* cfg_filename):
        tSimpleModule (cmdline, cfg_filename) {};

    protected:

        virtual bool process ();

        virtual bool register_sections (lib::tMemPool& trash_pool);

        virtual void unregister_sections ();

        virtual bool process_config (lib::tMemPool& conf_mp, lib::tMemPool&
                trash_pool);

        virtual void cleanup_config ();

    private:

        static struct lib::cfitem s_bingrep_section[];

};

#endif
