/**
 * @brief Echelog BinInfo tool
 *
 * This tool reads all messages from input bin and creates
 * informative summary
 *
 * @author Ales Privetivy <ales@ucw.cz>
 *
 */

#include "bininfo.h"

#define DEFAULT_CONFIG_FILE ECHELOG_CONF_DIR  "/common.conf"

using namespace lib;
using namespace messages;
using namespace spool;
using namespace mframes;

struct cfitem tBinInfoModule::s_bininfo_section[]={
    {name: "BinInfo",            type: CT_SECTION,       data: NULL},
    {name: "InputDatafile",      type: CT_STRING,        data: &input_data_file,
varf: NULL, varf_reset: NULL, flags: ITEM_REQUIRED },
    {name: "InputIndexfile",     type: CT_STRING,        data: &input_index_file,
varf: NULL, varf_reset: NULL, flags: ITEM_REQUIRED },
    {name: "OutputFile",         type: CT_STRING,        data: &output_file,
varf: NULL, varf_reset: NULL, flags: ITEM_REQUIRED },
    {name: NULL,                 type: CT_STOP,          data: NULL}
};

/* variables read from config file */
char *input_data_file, *input_index_file, *output_file;

/* input bin */
class tBinStore *inbin;
/* output instance to xml */
class tXMLOutput xml;
/* file descriptor of output file */
int xml_fd;

bool tBinInfoModule::process ()
{
    create_output ();

    return false;
}
        
bool tBinInfoModule::register_sections (lib::tMemPool& trash_pool)
{
    if (tSimpleModule::register_sections (trash_pool))
        return true;

    cf_register (s_bininfo_section);

    return false;
}

void tBinInfoModule::unregister_sections ()
{
    tSimpleModule::unregister_sections ();

    cf_unregister (s_bininfo_section[0].name);
}

bool tBinInfoModule::process_config (lib::tMemPool& conf_mp, lib::tMemPool&
        trash_pool)
{
    int tmp_fd;

    if (tSimpleModule::process_config (conf_mp, trash_pool))
        return true;

    /* create output xml file */
    if (!strcmp (output_file, "stdout"))
        xml_fd = 1;
    else
        xml_fd = s_open (output_file, O_WRONLY | O_CREAT | O_TRUNC,
                         S_IRUSR | S_IWUSR);
    if (xml_fd < 0)
    {
        log (L_ERROR, "Can't create file \"%s\" for bininfo output.", 
                      output_file);
        goto ancestor_clean;
    }

    /* setup the xml output object */
    xml.set_fd (xml_fd);
    xml.set_die_on_err (true);

    /* output the xml header */
    xml.print_header ("bininfo.xsl");

    /* check whether input data file exists */
    if ((tmp_fd=s_open (input_data_file, O_RDONLY))< 0)
    {
        xml_die ("Cannot open input BIN files.");
        goto output_file_clean;
    }

    s_close (tmp_fd);

    /* create input bin instance and open it for reading */
    inbin = new spool::tBinStore(0);
    if (!inbin)
    {
        xml_die ("Not enough memory.");
        goto output_file_clean;
    }

    if (inbin->open_r (input_data_file, input_index_file))
    {    
        xml_die ("Cannot open input BIN files.");
        goto in_binstore_clean;
    }

    return false;

in_binstore_clean:
    delete inbin;

output_file_clean:
    s_close (xml_fd);

ancestor_clean:
    tSimpleModule::cleanup_config ();

    return true;
}

void tBinInfoModule::cleanup_config ()
{
    inbin->close ();
    delete inbin;
    s_close (xml_fd);
    tSimpleModule::cleanup_config ();
}

/* --===  PROGRAM ENTRY POINT ===-- */
int main (int argc, char *argv[])
{
    tModule::tCmdLine cmdline (argc, argv);
    tBinInfoModule bininfo (cmdline, DEFAULT_CONFIG_FILE);

    if (bininfo.init ())
        return 1;

    int ret = bininfo.run ();
    bininfo.cleanup();
   
    return ret;
}
