/**
 * @file
 * @brief Header file for Echelog BinInfo
 *
 * @author Ales Privetivy <ales@ucw.cz>
 *
 */                                                                             

#ifndef ECHELOG_BININFO_H
#define ECHELOG_BININFO_H

#include "mframes/simplemodule.h"
#include "spool/binstore.h"
#include "messages/msgbuf.h"
#include "messages/msgheader.h"
#include "messages/msgchunk.h"
#include "messages/msgmgr.h"

#include <sys/types.h>
#include <sys/stat.h>

/* variables read from config file */
extern char *input_data_file, *input_index_file, *output_file;

/* input bin */
extern class spool::tBinStore *inbin;
/* output instance to xml */
extern class lib::tXMLOutput xml;
/* file descriptor of output file */
extern int xml_fd;

/* structure for message summary */
struct tMsgSummary {
    int msg_type;
    time_t first_time;
    time_t last_time;
    long space;
    long count;
};

/* structure for host summary */
struct tHostSummary {
    byte host[4];
    class lib::tList *ms;
    time_t first_time;
    time_t last_time;
    long space;
    long count;
};

/* memory pool for summary lists */
extern class lib::tMemPool list_mp;

/* list for message and host summary */
extern class lib::tList msgsum, hostsum;

/* statistics for all hosts together */
extern struct tHostSummary allhosts;

/// print error output
void xml_die(char *message);

/// create output
void create_output ();

/**
 * This class implements a typical miniagent's functionality.
 * The agent periodically reads informational chunks
 * from a supplied parser and forms them into messages which
 * are passed to the next module. It also covers the basic miniagent's
 * configuration and reconfiguration.
 *
 * This class should not have multiple instances in the program.
 */
class tBinInfoModule : public mframes::tSimpleModule
{
    public:
        tBinInfoModule (tCmdLine& cmdline, const char* cfg_filename):
        tSimpleModule (cmdline, cfg_filename) {};

    protected:

        virtual bool process ();

        virtual bool register_sections (lib::tMemPool& trash_pool);

        virtual void unregister_sections ();

        virtual bool process_config (lib::tMemPool& conf_mp, lib::tMemPool&
                trash_pool);

        virtual void cleanup_config ();

    private:

        static struct lib::cfitem s_bininfo_section[];

};

#endif
