/**
 * @file
 * @brief Header file for Echelog BinSelect
 *
 * @author Ales Privetivy <ales@ucw.cz>
 *
 */                                                                             

#ifndef ECHELOG_BINSELECT_H
#define ECHELOG_BINSELECT_H

#include "mframes/simplemodule.h"
#include "spool/binstore.h"
#include "messages/msgbuf.h"
#include "messages/msgheader.h"
#include "messages/msgchunk.h"
#include "messages/msgmgr.h"

#include <sys/types.h>
#include <sys/stat.h>

/**
 * This class implements a typical miniagent's functionality.
 * The agent periodically reads informational chunks
 * from a supplied parser and forms them into messages which
 * are passed to the next module. It also covers the basic miniagent's
 * configuration and reconfiguration.
 *
 * This class should not have multiple instances in the program.
 */
class tBinSelectModule : public mframes::tSimpleModule
{
    public:
        tBinSelectModule (tCmdLine& cmdline, const char* cfg_filename):
        tSimpleModule (cmdline, cfg_filename) {};

    protected:

        virtual bool process ();

        virtual bool register_sections (lib::tMemPool& trash_pool);

        virtual void unregister_sections ();

        virtual bool process_config (lib::tMemPool& conf_mp, lib::tMemPool&
                trash_pool);

        virtual void cleanup_config ();

    private:

        static struct lib::cfitem s_binselect_section[];

};

/* placeholders for parameters */
extern char *starttime, *endtime, *hostname, *agentname, *instancename;
extern char *input_data_file, *input_index_file, *output_file, *msgname, *out_attrs;

/* we will output contents of messages within this interval */
extern time_t start_time, end_time;

/* we will output contents only fot this host */
extern byte *host;

/* we will output contents only of this message */
extern int msg_type;

/* instance of input bin */
extern class spool::tBinStore *inbin;

/* file descriptor of output file */
extern int out_fd;

/* list of attributes we will output */
extern class lib::tList attrs_list;

/// Parse and register types and messages from configuration file
void load_config (int argc, char *argv[]);

#endif
