/**
 * @file
 * @brief Header file for Echelog BinSnapshot
 *
 * @author Ales Privetivy <ales@ucw.cz>
 *
 */                                                                             

#ifndef ECHELOG_BINSNAPSHOT_H
#define ECHELOG_BINSNAPSHOT_H

#include "mframes/simplemodule.h"
#include "spool/binstore.h"
#include "messages/msgbuf.h"
#include "messages/msgheader.h"
#include "messages/msgchunk.h"
#include "messages/msgmgr.h"

#include <sys/types.h>
#include <sys/stat.h>

/**
 * This class implements a typical miniagent's functionality.
 * The agent periodically reads informational chunks
 * from a supplied parser and forms them into messages which
 * are passed to the next module. It also covers the basic miniagent's
 * configuration and reconfiguration.
 *
 * This class should not have multiple instances in the program.
 */
class tBinSnapshotModule : public mframes::tSimpleModule
{
    public:
        tBinSnapshotModule (tCmdLine& cmdline, const char* cfg_filename):
        tSimpleModule (cmdline, cfg_filename) {};

    protected:

        virtual bool process ();

        virtual bool register_sections (lib::tMemPool& trash_pool);

        virtual void unregister_sections ();

        virtual bool process_config (lib::tMemPool& conf_mp, lib::tMemPool&
                trash_pool);

        virtual void cleanup_config ();

    private:

        static struct lib::cfitem s_binsnapshot_section[];

};

/* time at when the snapshot should be generated */
extern time_t snap_time, time_used;
/* source of message */
extern char *agent, *instance;
extern byte *host;
/* message which state we restore */
extern int msg_type;
/* input file names */
extern char *input_data_file, *input_index_file, *output_file, *msgname;
extern char tmp_data_file[], tmp_index_file[];

/* temporary and input binstore parameters */
extern class spool::tBinStore *outbin, *inbin;

/* output snapshot file */
extern class lib::tXMLOutput xml;
/* file descriptor of output snapshot file */
extern int xml_fd;

/// print error output
void xml_die(char *message);

/// print beginning of document
void xml_info ();

void generate_output ();

#endif
