/**
 * @brief Supplementary Routines
 * 
 * @author Kamil Toman <ktoman@email.cz>
 *
 */                                                                             

#include "lib/lib.h"
#include "cmd_param.h"
#include <string.h>


using namespace lib;


// Serialize the three input non-empty strings as null separated items
// into a newly allocated buffer.
long cmd_store_strings (char **ret, const char *c1, const char *c2,
        const char *c3, long limit)
{
    ASSERT (ret && c1 && c2 && c3);

    /* Compute string lengths including the terminating NULL */
    long c1_len = strlen (c1) + 1, c2_len = strlen (c2) + 1, 
        c3_len = strlen (c3) + 1, total_len = c1_len + c2_len + c3_len;

    /* For empty string return a NULL pointer */
    if (total_len == 3)
    {
        *ret = NULL;
        return 0;
    }

    /* Fail, if the result would exceed the buffer limit */
    if (c1_len + c2_len + c3_len > limit)
        return -1;


    /* Concatenate the strings as null separated items */
    *ret = static_cast<char *> (xmalloc (c1_len + c2_len + c3_len));
    memcpy (*ret, c1, c1_len);
    memcpy (*ret + c1_len, c2, c2_len);
    memcpy (*ret + c1_len + c2_len, c3, c3_len);

    /* Return the length of resulting buffer */
    return total_len;
}


// Retrieve the second two null-separated string from the input buffer
void cmd_retrieve_strings (char **c1, char **c2, const char *src, 
        const long src_len)
{
    long c1_len, c2_len, len = src_len;

    ASSERT (c1 && c2);

    /* Initialize the strings to be returned */
    *c1 = NULL;
    *c2 = NULL;

    /* If there is nothing to retrieve let them as null pointers */
    if (src_len <= 0)
        return;

    ASSERT (src);

    /* Skip over the first item which can be trivially accessed */
    while (len-- && *src++) 
        ;

    /* Compute the lengths of other stored items */
    c1_len = s_strnlen (src, len) + 1;
    c2_len = s_strnlen (src + c1_len, len - c1_len) + 1;

    if (c1_len > 1)
    {
        /* If the second item is set return a copy */
        *c1 = static_cast<char *> (xmalloc (c1_len));
        memcpy (*c1, src, c1_len);
    }

    if (c2_len > 1)
    {
        /* If the third item is set return a copy */
        *c2 = static_cast<char *> (xmalloc (c2_len));
        memcpy (*c2, src + c1_len, c2_len);
    }
}
