/**
 * @file
 * @brief Echelog Overseer Remote Control
 *
 * This module contains an interface which allows a user to start, stop, 
 * restart or reconfigure any particular module or the whole echelog 
 * system as such.
 *
 * @author Kamil Toman <ktoman@email.cz>
 *
 */

#include "echelogctl.h"
#include "socks/unixsocket.h"
#include "socks/ctrl_unixsocket.h"


#ifndef DOXY_SKIP
using namespace lib;
using namespace socks;
#endif


/// Echelogctl main socket manager
static tSocketMgr smgr;


/// Default data processing socket handler
void process_f(class tSocket *sock)
{
    tCtrlUNIXSocket *s = static_cast<tCtrlUNIXSocket *> (sock);
    
    if (s->receive_cmd ())
        log (L_WARN, "Error receiving data from the control socket.");

    log (L_WARN, "Discarding the control command received from echelogd.");
}


/* --===  PROGRAM ENTRY POINT ===-- */
int main (int argc, char *argv[])
{
    tCtrlUNIXSocket *ctrl = new tCtrlUNIXSocket;
    tCommand *c;

    /* Init logging routines */
    argv[0] = "echelogctl";
    log_init (argv[0]);

    if (!ctrl)
        die ("Not enough memory.");

    /* Process the control command line structure */
    c = process_cmdline (argc, argv);

    /* Setup the default control socket handlers */
    ctrl->setup_defaults (argv[0]);

    /* Check if echelog is already running */
    if (!ctrl->register_echelog_module (smgr))
    {
        debug_log ('t', "Control module successfully connected.");

        /* Yes, try to send admin's command to it */
        ctrl->store_cmd (c);
        free_cmd (c);
        if (ctrl->send_cmd () < 0)
            die ("Can't send command to echelogd");
    }
    else
    {
        /* Free the command and abort */
        free_cmd (c);
        ctrl->close_socket ();
        die ("Can't communicate with echelogd.");
    }
    
    /* Tidy up and quit */
    smgr.cleanup ();
    log_close ();
    return 0;
}
