/**
 * @file
 * @brief Echelog Miniagent -- getlog
 *
 * This agent periodically reads specified logs and passes information
 * stored there as echelog messages.
 *
 * @author Kamil Toman <ktoman@email.cz>
 *
 */

#ifndef ECHELOG_MAGENT_GETLOG_BUFFER_H
#define ECHELOG_MAGENT_GETLOG_BUFFER_H

#include "filebuffer.h"


/// Buffer above all log file caches
class tBuffer {

    private:

        int num_of_buffers;             ///< Number of buffer caches
        tFileBuffer *current_buffer;    ///< Current file cache
        tFileBuffer *buffers;           ///< Array of all file caches
        char *state_dir;                ///< Directory for state files

        /** Advance to next file cache and return true on rotation */
        inline bool next_buffer ();

    public:

        /** Create an empty log buffer */
        tBuffer () : num_of_buffers (0), current_buffer (0), buffers (NULL),
            state_dir (NULL) {};
        /** Delete the log buffer */
        ~tBuffer () { unregister_files(); }

        /** Load state files for all file caches */
        void load_state_files ();
        /** Save state files for all file caches */
        void save_state_files ();
        /** Register filenames for all file caches */
        void register_files (const char *conf_state_dir, lib::tList &filenames);
        /** Unregister all log files */
        void unregister_files ();
        /** Get a single line from a file cache (preserve the order specified
         * in getlog's configuration */
        void get_line (tBufferResponse &r);
};


#endif
