/**
 * @file
 * @brief Echelog Miniagent -- getlog
 *
 * @module getlog configuration module
 *
 * This agent periodically reads specified logs and passes information
 * stored there as echelog messages.
 *
 * @author Kamil Toman <ktoman@email.cz>
 *
 */

#include "getlog.h"

using namespace lib;
using namespace messages;
using namespace magents;

char *tMagentGetlog::s_state_dir = NULL;
struct cfitem tMagentGetlog::s_getlog_section[]={
    {name: "GetLog",            type: CT_SECTION,       data: NULL},
    {name: "StateDir",          type: CT_STRING,        data: &s_state_dir,
        varf: NULL, varf_reset: NULL, flags: ITEM_REQUIRED },
    {name: "LogFile",           type: CT_LIST,          data: NULL,
        varf: NULL, varf_reset: NULL, flags: ITEM_REQUIRED },
    {name: NULL,                type: CT_STOP,          data: NULL}
};


bool tMagentGetlog::static_init ()
{
    if (tMagent::static_init ())
        goto fail;

    chunk = new tMsgChunk (SIMPLE_LOG_MSG, mp);
    if (!chunk) 
    {
        log (L_ERROR, "Not enough memory.");
        goto fail;
    }

    log_line = static_cast<tVarchar *>(chunk->get_attr("LINE"));
    if (!log_line) 
    {
        log (L_ERROR, "Can't create a new chunk.");
        goto anc_clean;
    }

    chunk->set_status (ST_ADDED);

    m_inited = true;
    return false;

anc_clean:
    tMagent::static_cleanup ();
fail:
    m_inited = false;
    return true;
}


void tMagentGetlog::static_cleanup ()
{
    if (log_line) 
    {
        delete log_line;
        log_line = NULL;
    }

    if (chunk)
    {
        delete chunk;
        chunk = NULL;
    }
    
    tMagent::static_cleanup ();
}


bool tMagentGetlog::register_sections (lib::tMemPool& trash_pool)
{
    if (tMagent::register_sections (trash_pool))
        return true;

    tList* list = new tList (trash_pool);
    if (list == NULL)
    {
        log (L_ERROR, "Not enough memory.");
        return true;
    }
    s_getlog_section[2].data = list;

    cf_register(s_getlog_section);
    
    return false;
}


void tMagentGetlog::unregister_sections ()
{
    cf_unregister (s_getlog_section[0].name);

    delete static_cast<lib::tList *> (s_getlog_section[2].data);
    s_getlog_section[2].data = NULL;

    tMagent::unregister_sections ();
}


bool tMagentGetlog::process_config (lib::tMemPool& conf_mp, 
        lib::tMemPool& trash_pool)
{
    tList *filelist;

    /* call ancestor's processing */
    if (tMagent::process_config (conf_mp, trash_pool))
        goto fail;

    filelist = static_cast<tList *> (s_getlog_section[2].data);
    if (!filelist)
    {
        log (L_ERROR, "Error processing input log filenames.");
        goto anc_clean;
    }

    buffer.register_files (s_state_dir, *filelist);
    return false;

anc_clean:
    tMagent::cleanup_config ();
fail:
    return true;

}


void tMagentGetlog::cleanup_config ()
{
    buffer.unregister_files ();

    /* call ancestor's cleanup */
    tMagent::cleanup_config ();

    s_state_dir = NULL;
}


bool tMagentGetlog::dynamic_init ()
{
    if (tMagent::dynamic_init ())
        goto fail;

    buffer.load_state_files ();
    return false;

fail:
    return true;
}
