/**
 * @file
 * @brief Echelog Miniagent -- getlog
 *
 * This agent periodically reads specified logs and passes information
 * stored there as echelog messages.
 *
 * @author Kamil Toman <ktoman@email.cz>
 *
 */

#ifndef ECHELOG_MAGENT_GETLOG_GETLOG_H
#define ECHELOG_MAGENT_GETLOG_GETLOG_H

#include "messages/msgmgr.h"
#include "magents/magent.h"
#include "buffer.h"

/** Log messages sent by GetLog */
#define SIMPLE_LOG_MSG     "SimpleLog"

/** Suffix for rotated log files */
#define ROTATE_SUFFIX      ".1"
/** Length of ROTATE_SUFFIX */
#define ROTATE_SUFFIX_LEN  2


class tMagentGetlog : public magents::tMagent {

    private:

        //-------------------------------
        // Owned components
        //-------------------------------

        tBuffer buffer;                 ///< Round file buffer
        lib::tMemPool mp;               ///< Magent's internal mempool
        messages::tMsgChunk *chunk;     ///< Single log chunk
        messages::tVarchar *log_line;   ///< Single log line

        /// State files directory
        static char *s_state_dir;

        /// configuration parameters
        static struct lib::cfitem s_getlog_section[];
        
        //------------------------------------------------
        // Chunk handling functions
        //------------------------------------------------             

        /// store reponse data to chunk
        inline long prepare_chunk (tBufferResponse &r);
        /// get chunk type
        inline int get_msg_type ();
        /// send the last unsent message
        inline void send_last_message ();
        /// sent the last non-empty unsent message
        inline void send_last_non_empty_message ();
        /// sent empty full state snapshot
        inline void send_noninc_snapshot ();
        /// sent alert message
        inline void send_alert (char *alert_str);

    public:

        /// Getlog constructor
        tMagentGetlog (tMagentGetlog::tCmdLine& cmdline, 
                const char* cfg_filename) : tMagent (cmdline, cfg_filename), 
                mp (1024), chunk (NULL), log_line (NULL) {}
        
        //------------------------------------------------
        // Implementation of tMagent interface
        //------------------------------------------------             

        virtual bool process_messages (bool& on_time);
        
        //------------------------------------------------
        // Implementation of tReconfigurableModule interface
        //------------------------------------------------             

        virtual bool static_init ();
        virtual void static_cleanup ();
        virtual bool dynamic_init ();

        //------------------------------------------------
        // Implementation of tConfigurable interface
        //------------------------------------------------             
     
        virtual bool register_sections (lib::tMemPool& trash_pool);
        virtual void unregister_sections ();
        virtual bool process_config (lib::tMemPool& conf_mp, 
            lib::tMemPool& trash_pool);
        virtual void cleanup_config ();
};


/* Here we actually define all tMagentEchelog inline functions */
#include "getlog.hxx"

#endif

