/**
 * @brief Echelog Miniagent - getlog
 *
 * @module getlog message handling module 
 *
 * This agent periodically reads specified logs and passes information
 * stored there as echelog messages.
 *
 * @author Kamil Toman <ktoman@email.cz>
 *
 */                                                                             

#ifndef ECHELOG_MAGENT_GETLOG_GETLOG_HXX
#define ECHELOG_MAGENT_GETLOG_GETLOG_HXX


inline long tMagentGetlog::prepare_chunk (tBufferResponse &r)
{
    /* implant the string portion */
    log_line->set_direct_value (r.str, r.str_length);

    return chunk->get_store_length ();
}


inline int tMagentGetlog::get_msg_type ()
{
    static int type = messages::gb_message_manager.get_message_type_by_handle
        (messages::gb_message_manager.get_handle_by_name(SIMPLE_LOG_MSG));

    return type;
}


inline void tMagentGetlog::send_last_message ()
{
    if (!mp_mf->get_num_of_records())
        mp_mf->set_incremental_message (true);
    mp_mf->send_message (get_msg_type ());
    buffer.save_state_files ();
}


inline void tMagentGetlog::send_last_non_empty_message ()
{
    if (mp_mf->get_num_of_records() > 0) 
    {
        mp_mf->send_message (get_msg_type ());
        buffer.save_state_files ();
    }
}


inline void tMagentGetlog::send_noninc_snapshot ()
{
    mp_mf->set_incremental_message (false);
    mp_mf->send_message (get_msg_type ());
    mp_mf->set_incremental_message (true);
}


inline void tMagentGetlog::send_alert (char *alert_str)
{
    send_last_non_empty_message ();
    mp_mf->send_alert (alert_str);
    lib::xfree (alert_str);
}


#endif
