/**
 * @file
 * @brief Contains declaration of tMsgSniffPrinter class
 *
 * @author Kamil Toman <ktoman@email.cz>, Marek Zoth <marek.zoth@volny.cz>
 *
 */

#ifndef ECHELOG_MSGSNIFFPRINTER_H
#define ECHELOG_MSGSNIFFPRINTER_H

#include "messages/msgbuf.h"
#include "messages/msgchunk.h"

/**
 * @brief TODOC:
 */
class tMsgSniffPrinter
{
    public:
        //-------------------------------
        // Interface
        //-------------------------------

        /// Constructor
        tMsgSniffPrinter ();
        /// Destructor
        virtual ~tMsgSniffPrinter ();

        /**
         * @brief TODOC:
         *
         * @return
         * If succesful returns false. On failure return true.
         */
        virtual bool init (FILE* dest);

        /**
         * @brief Cleanup of the printer
         *
         * Releases and resets all allocated resources in init().
         * It is the reverse function for init().
         */
        virtual void cleanup ();

        /**
         * @brief TODOC:
         */
        virtual bool print_message (messages::tMsgBuf& msg);

    protected:
        //-------------------------------
        // Internal methods
        //-------------------------------

        /**
         * @brief TODOC:
         */
        virtual void print_header (messages::tMsgBuf &message);

        /**
         * @brief TODOC:
         */
        virtual void printout (lib::tList &list);

        /**
         * @brief TODOC:
         */
        virtual void print_chunk (messages::tMsgChunk &chunk, int num);

        /**
         * @brief TODOC:
         */
        static const char* tMsgSniffPrinter::str_status (
            messages::tMSG_STATUS status);

        //-------------------------------
        // Owned components
        //-------------------------------

        /// destination file to print to
        FILE* m_logfile;

        lib::tMemPool m_chunk_mp;
        lib::tMemPool m_tmp_pool;
        lib::tList m_tmp_list;

    private:
        //-------------------------------
        // State attributes
        //-------------------------------

        /// true if succesfully initialized
        bool m_inited;
};

#endif

