/**
 * @brief Echelog Receiver
 *
 * @author Ales Privetivy <ales@ucw.cz>
 *
 */                                                                             

#include "receiver.h"
#include "socks/unixsocket_nb.h"
#include "socks/ctrl_unixsocket_nb.h"

#include <unistd.h>
#include <signal.h>
#include <time.h>
#include <sys/time.h>


using namespace lib;
using namespace socks;


/* signal states */
static volatile bool sig_exit_received = false,
            sig_reload_received = false;

/* trivial signal handling function */
void sig_handling_f (int signum)
{
    switch (signum)
    {
        case SIGHUP:
            sig_reload_received = true;
            break;
        case SIGINT:
        case SIGQUIT:
        case SIGPIPE:
            sig_exit_received = true;
    }
}

/* signal handling function instalation */
void install_sig_handlers ()
{
    set_signal_handler (SIGHUP, sig_handling_f);
    set_signal_handler (SIGINT, sig_handling_f);
    set_signal_handler (SIGQUIT, sig_handling_f);
    set_signal_handler (SIGPIPE, sig_handling_f);
}


/* --===  PROGRAM ENTRY POINT ===-- */
int main (int argc, char *argv[])
{
    tCtrlUNIXSocketNB ctrl;
  
    /* init logging routines */
    log_init (argv[0]);

    /* install signal handlers */
    install_sig_handlers ();
   
    /* register types and messages from configuration file */
    load_config (argc, argv);

    /* setup the control channel */
    ctrl.setup_defaults (argv[0]);
    if (!ctrl.register_echelog_module (*smgr))
        log (L_INFO, "Successfully registered at echelogd.");

    for (;;)    
    {
        /* process data on sockets */
        smgr -> process();

        /* just exit if the user wishes so */
        if (sig_exit_received)
        {
            log(L_INFO, "Termination request received, attempting to exit "
                    "gracefully.");
            /* and bail out */
            break;
        }
        /* if scheduled do the configuration reload */
        if (sig_reload_received)
        {
            reread_configuration ();
            sig_reload_received = false;
        }

    }

    /* release all socket held in socket manager */
    smgr -> cleanup();

    /* remove listenning socket */
    in_sock -> close_socket();
    delete in_sock;

    /* remove outgoing socket */
    out_sock -> close_socket();
    delete out_sock;

    /* write all state info */
    host_mgr -> write_all_states();

    /* Exit */
    log (L_INFO, "Successfully terminated.");
    log_close();

    return 0;
}
