/**
 * @file
 * @brief Echelog Sender  
 *
 * @author Ales Privetivy <ales@ucw.cz>
 *
 */                                                                             

#ifndef ECHELOG_RECEIVER_H
#define ECHELOG_RECEIVER_H

#include "socks/unixsocket.h"
#include "socks/sslsocket.h"
#include "statusmsg.h"


#define DEFAULT_CONFIG_FILE DEFAULT_MODULE_CONFIG
#define REC_STATE_BUFF_SIZE 12

using namespace socks;

extern char *pk_file, *cert_file, *root_file, *server_name, *dh_file;
extern char *state_dir;
extern int r_port;
extern class tHostManager *host_mgr;
extern class tUNIXSocket *out_sock;
extern class tSSLSocket *in_sock;
extern class tSocketMgr *smgr;

void unix_sock_err_f(class tSocket *sock);
void unix_sock_hup_f(class tSocket *sock);
void unix_sock_read_f(class tSocket *sock);
void ssl_sock_tcp_err(class tSocketNB *sock);
void ssl_sock_tcp_hup(class tSocketNB *sock);
void ssl_sock_connected(class tSocketNB *sock);
void ssl_sock_read_available(class tSocketNB *sock);
void ssl_sock_read_finished(class tSocketNB *sock);
void ssl_sock_write_finished(class tSocketNB *sock);
void ssl_sock_shutdown_finished(class tSSLSocket *sock);

enum tRState {
    RS_EMPTY,
    RS_NEGOTIATE,
    RS_RECEIVE,
    RS_COMMIT,
    RS_QUIT
             };

enum tQuitStatus {
    QS_OK,
    QS_UNKNOWN,
    QS_ALREADY_CONNECTED,
    QS_INV_CERT,
    QS_QUOTA_EXC
                 };

struct tHostInfo {
    char *hostname;
    char *common_name;
    char *filename;
    spool::tIdentificator received;
    int connected;
    int32_t start_time;
    long period;
    long limit;
    int32_t flow;
};

struct tRecAppData {
        class tStateInfo state_info;
        char data_buffer[MAX_MESSAGE_SIZE+16];
        int handle;
        enum tRState phase;
};

class tHostManager {

    private:
      int allowed_hosts_cnt;
      struct tHostInfo *allowed_hosts;
      char *state_dir;
      class lib::tMemPool *mp;

      int check_file ( int handle );

    public:
      tHostManager ( char *st_dir );
      ~tHostManager ();
      
      int set_allowed_hosts( class lib::tList &hl );
      int get_host_handle ( char *hostname );
      
      int read_state ( int handle );
      int write_state ( int handle );
      
      spool::tIdentificator get_state( int handle )
      { return allowed_hosts[handle].received; }

      void set_state( int handle, spool::tIdentificator state )
      { allowed_hosts[handle].received = state; }

      int get_connected ( int handle )
      { return allowed_hosts[handle].connected; }
      
      void set_connected ( int handle , int flag )
      { allowed_hosts[handle].connected = flag; }

      struct tHostInfo * get_host_info (int handle)
      { return allowed_hosts+handle; }
      
      void write_all_states();
      
};

/// Parse and register types and messages from configuration file
void load_config (int argc, char *argv[]);
void reread_configuration ();
void bring_down();

#endif
