/**
 * @file
 * @brief Echelog Receiver 
 *
 * @author Ales Privetivy <ales@ucw.cz>
 *
 */                                                                             

#ifndef ECHELOG_STATUSMSG_H
#define ECHELOG_STATUSMSG_H

#include "messages/testbuf.h"
#include "spool/pagefile.h"

#define STATE_MESSAGE_SIZE 2*sizeof(long)

enum tCommitState {
        CS_START=1,
        CS_REQ=2,
        CS_ACCEPTED=3,
        CS_REFUSED=4,
        CS_QUIT=5    
};

using messages::tPlainBuf;

class tStateInfo {

    private:
      char msg_buf[STATE_MESSAGE_SIZE];
      SIMPLE_PROPERTY( enum tCommitState, state );
      SIMPLE_PROPERTY( spool::tIdentificator, arg );

    public:
      inline int store_message();
      inline int load_message();
      inline char *get_msg_buf();
      
};

inline int tStateInfo::store_message()
{
     using messages::tPlainBuf;
     tPlainBuf b( msg_buf, STATE_MESSAGE_SIZE );

     STORE_INT32 ( &b, (long) state );
     STORE_INT32 ( &b, (long) arg );

     return 0;
}

inline int tStateInfo::load_message() 
{
     using messages::tPlainBuf;
     tPlainBuf b( msg_buf, STATE_MESSAGE_SIZE );
     int32_t tmp;

     READ_INT32 ( &b, tmp );
     state = (enum tCommitState) tmp;
     READ_INT32 ( &b, tmp );
     arg = (spool::tIdentificator) tmp;

     return 0;
}

inline char *tStateInfo::get_msg_buf()
{
     return msg_buf;
}

#endif
