/**
 * @file
 * @brief Echelog Sender  
 *
 * @author Ales Privetivy <ales@ucw.cz>
 *
 */                                                                             

#ifndef ECHELOG_RECEIVER_H
#define ECHELOG_RECEIVER_H

#include "spool/pagefile.h"
#include "spool/queue.h"
#include "socks/unixsocket.h"
#include "socks/sslsocket.h"

#define DEFAULT_CONFIG_FILE DEFAULT_MODULE_CONFIG
#define REC_STATE_BUFF_SIZE 12 
#define MAX_ITEM_ID         1000000000

using namespace socks;
using namespace spool;

extern char *pk_file, *cert_file, *root_file, *server_name;
extern char *state_dir, *sp_dir, *idx_file, *common_name;
extern int r_port, wr_thrg, ta_secs, compress;
extern long flags;
extern class tUNIXSocket *in_sock;
extern class tSSLSocket *out_sock;
extern class tSocketMgr *smgr;
extern class tQueue *queue;
extern class tSSLContext *ctx;

void unix_sock_err_f(class tSocket *sock);
void unix_sock_hup_f(class tSocket *sock);
void unix_sock_read_f(class tSocket *sock);
void ssl_sock_tcp_err(class tSocketNB *sock);
void ssl_sock_tcp_hup(class tSocketNB *sock);
void ssl_sock_connected(class tSocketNB *sock);
void ssl_sock_read_available(class tSocketNB *sock);
void ssl_sock_read_finished(class tSocketNB *sock);
void ssl_sock_write_finished(class tSocketNB *sock);
void ssl_sock_shutdown_finished(class tSSLSocket *sock);

enum tRState {
    RS_EMPTY,
    RS_NEGOTIATE,
    RS_SEND,
    RS_COMMIT,
    RS_QUIT
             };

enum tQuitStatus {
    QS_OK,
    QS_UNKNOWN,
    QS_ALREADY_CONNECTED,
    QS_INV_CERT,
    QS_QUOTA_EXC
                 };


/// Parse and register types and messages from configuration file
void load_config (int argc, char *argv[]);
void reread_configuration ();
void bring_down();
void try_write();
// Functions for renumbering support
int shift_to_start(char *old_spool_dir, char *temp_spool_dir, char *idx_file);
int tag_renumber(char *spooldir);
int untag_renumber(char *spooldir);
int istag_renumber(char *spooldir);

#endif
